from ...models.workflow_state_escalation_models import WorkflowStateEscalation
from ...tasks import (
    task_workflow_instance_check_escalation,
    task_workflow_instance_check_escalation_all
)

from ..literals import (
    TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_AMOUNT,
    TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_AMOUNT_EDITED,
    TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_COMMENT,
    TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_UNIT
)

from .workflow_template_transition_mixins import WorkflowTemplateTransitionTestMixin


class WorkflowTemplateStateEscalationTestMixin(
    WorkflowTemplateTransitionTestMixin
):
    _test_object_model = WorkflowStateEscalation
    _test_object_name = '_test_workflow_template_state_escalation'

    def _create_test_workflow_template_state_escalation(self, extra_kwargs=None):
        kwargs = {
            'amount': TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_AMOUNT,
            'comment': TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_COMMENT,
            'transition': self._test_workflow_template_transition,
            'unit': TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_UNIT
        }

        if extra_kwargs:
            kwargs.update(extra_kwargs)

        self._test_workflow_template_state_escalation = self._test_workflow_template_states[0].escalations.create(
            **kwargs
        )


class WorkflowTemplateStateEscalationAPIViewTestMixin(
    WorkflowTemplateStateEscalationTestMixin
):
    def _request_test_workflow_template_state_escalation_create_api_view(self):
        data = {
            'amount': 1,
            'enabled': True,
            'workflow_template_transition_id': self._test_workflow_template_transition.pk
        }

        self._test_object_track()

        response = self.post(
            viewname='rest_api:workflow-template-state-escalation-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_states[0].pk
            }, data=data
        )

        self._test_object_set()

        return response

    def _request_test_workflow_template_state_escalation_delete_api_view(self):
        return self.delete(
            viewname='rest_api:workflow-template-state-escalation-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_states[0].pk,
                'workflow_template_state_escalation_id': self._test_workflow_template_state_escalation.pk
            }
        )

    def _request_test_workflow_template_state_escalation_detail_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-escalation-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_states[0].pk,
                'workflow_template_state_escalation_id': self._test_workflow_template_state_escalation.pk
            }
        )

    def _request_test_workflow_template_state_escalation_edit_via_patch_api_view(self, extra_data=None):
        data = {
            'amount': TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_AMOUNT_EDITED
        }

        if extra_data is not None:
            data.update(extra_data)

        return self.patch(
            viewname='rest_api:workflow-template-state-escalation-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_states[0].pk,
                'workflow_template_state_escalation_id': self._test_workflow_template_state_escalation.pk
            }, data=data
        )

    def _request_test_workflow_template_state_escalation_edit_via_put_api_view(self, extra_data=None):
        data = {
            'amount': TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_AMOUNT_EDITED,
            'workflow_template_transition_id': self._test_workflow_template_transitions[0].pk
        }

        if extra_data is not None:
            data.update(extra_data)

        return self.put(
            viewname='rest_api:workflow-template-state-escalation-detail', kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_states[0].pk,
                'workflow_template_state_escalation_id': self._test_workflow_template_state_escalation.pk
            }, data=data
        )

    def _request_test_workflow_template_state_escalation_list_api_view(self):
        return self.get(
            viewname='rest_api:workflow-template-state-escalation-list',
            kwargs={
                'workflow_template_id': self._test_workflow_template.pk,
                'workflow_template_state_id': self._test_workflow_template_states[0].pk
            }
        )


class WorkflowTemplateStateEscalationTaskTestMixin(
    WorkflowTemplateStateEscalationTestMixin
):
    def _execute_task_workflow_instance_check_escalation(
        self, test_workflow_instance_id
    ):
        task_workflow_instance_check_escalation.apply_async(
            kwargs={'workflow_instance_id': test_workflow_instance_id}
        ).get()

    def _execute_task_workflow_instance_check_escalation_all(self):
        task_workflow_instance_check_escalation_all.apply_async().get()


class WorkflowTemplateStateEscalationViewTestMixin(
    WorkflowTemplateStateEscalationTestMixin
):
    def _request_test_workflow_template_state_escalation_create_view(self, extra_data=None):
        data = {
            'transition': self._test_workflow_template_transition.pk
        }

        if extra_data is not None:
            data.update(extra_data)

        self._test_object_track()

        response = self.post(
            viewname='document_states:workflow_template_state_escalation_create',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_states[0].pk
            }, data=data
        )

        self._test_object_set()

        return response

    def _request_test_workflow_template_state_escalation_delete_view(self):
        return self.post(
            viewname='document_states:workflow_template_state_escalation_delete',
            kwargs={
                'workflow_template_state_escalation_id': self._test_workflow_template_state_escalation.pk
            }
        )

    def _request_test_workflow_template_state_escalation_edit_view(self, extra_data=None):
        data = {
            'amount': TEST_WORKFLOW_TEMPLATE_STATE_ESCALATION_AMOUNT_EDITED,
            'transition': self._test_workflow_template_state_escalation.transition.pk
        }

        if extra_data is not None:
            data.update(extra_data)

        return self.post(
            viewname='document_states:workflow_template_state_escalation_edit',
            kwargs={
                'workflow_template_state_escalation_id': self._test_workflow_template_state_escalation.pk
            }, data=data
        )

    def _request_test_workflow_template_state_escalation_list_view(self):
        return self.get(
            viewname='document_states:workflow_template_state_escalation_list',
            kwargs={
                'workflow_template_state_id': self._test_workflow_template_states[0].pk
            }
        )



from mayan.apps.testing.tests.mixins import TestMixinObjectCreationTrack

from ..models import WebLink

from .literals import (
    TEST_WEB_LINK_LABEL, TEST_WEB_LINK_LABEL_EDITED, TEST_WEB_LINK_TEMPLATE
)


class WebLinkTestMixin(TestMixinObjectCreationTrack):
    _test_object_model = WebLink
    _test_object_name = '_test_web_link'

    def _create_test_web_link(self, add_test_document_type=False):
        self._test_web_link = WebLink.objects.create(
            label=TEST_WEB_LINK_LABEL, template=TEST_WEB_LINK_TEMPLATE,
        )
        if add_test_document_type:
            self._test_web_link.document_types.add(self._test_document_type)


class DocumentTypeAddRemoveWebLinkViewTestMixin(WebLinkTestMixin):
    def _request_test_document_type_web_link_add_remove_get_view(self):
        return self.get(
            viewname='web_links:document_type_web_links', kwargs={
                'document_type_id': self._test_document_type.pk
            }
        )

    def _request_test_document_type_web_link_add_view(self):
        return self.post(
            viewname='web_links:document_type_web_links', kwargs={
                'document_type_id': self._test_document_type.pk
            }, data={
                'available-submit': 'true',
                'available-selection': self._test_web_link.pk
            }
        )

    def _request_test_document_type_web_link_remove_view(self):
        return self.post(
            viewname='web_links:document_type_web_links', kwargs={
                'document_type_id': self._test_document_type.pk
            }, data={
                'added-submit': 'true',
                'added-selection': self._test_web_link.pk
            }
        )


class ResolvedWebLinkAPIViewTestMixin(WebLinkTestMixin):
    def _request_resolved_web_link_detail_api_view(self):
        return self.get(
            viewname='rest_api:resolved_web_link-detail',
            kwargs={
                'document_id': self._test_document.pk,
                'resolved_web_link_id': self._test_web_link.pk
            }
        )
        self._create_test_web_link(add_test_document_type=True)

    def _request_resolved_web_link_list_api_view(self):
        return self.get(
            viewname='rest_api:resolved_web_link-list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_resolved_web_link_navigate_api_view(self):
        return self.get(
            viewname='rest_api:resolved_web_link-navigate',
            kwargs={
                'document_id': self._test_document.pk,
                'resolved_web_link_id': self._test_web_link.pk
            }
        )


class WebLinkAPIViewTestMixin(WebLinkTestMixin):
    def _request_test_web_link_create_api_view(self):
        self._test_object_track()

        response = self.post(
            viewname='rest_api:web_link-list', data={
                'label': TEST_WEB_LINK_LABEL,
                'template': TEST_WEB_LINK_TEMPLATE
            }
        )

        self._test_object_set()

        return response

    def _request_test_web_link_delete_api_view(self):
        return self.delete(
            viewname='rest_api:web_link-detail', kwargs={
                'web_link_id': self._test_web_link.pk
            }
        )

    def _request_test_web_link_detail_api_view(self):
        return self.get(
            viewname='rest_api:web_link-detail', kwargs={
                'web_link_id': self._test_web_link.pk
            }
        )

    def _request_test_web_link_edit_via_patch_api_view(self):
        return self.patch(
            viewname='rest_api:web_link-detail',
            kwargs={'web_link_id': self._test_web_link.pk}, data={
                'label': TEST_WEB_LINK_LABEL_EDITED
            }
        )

    def _request_test_web_link_edit_via_put_api_view(self):
        return self.put(
            viewname='rest_api:web_link-detail',
            kwargs={'web_link_id': self._test_web_link.pk}, data={
                'label': TEST_WEB_LINK_LABEL_EDITED,
                'template': TEST_WEB_LINK_TEMPLATE
            }
        )


class WebLinkDocumentTypeAPIViewMixin(WebLinkTestMixin):
    def _request_test_web_link_document_type_add_api_view(self):
        return self.post(
            viewname='rest_api:web_link-document_type-add',
            kwargs={'web_link_id': self._test_web_link.pk}, data={
                'document_type': self._test_document_type.pk
            }
        )

    def _request_test_web_link_document_type_list_api_view(self):
        return self.get(
            viewname='rest_api:web_link-document_type-list', kwargs={
                'web_link_id': self._test_web_link.pk
            }
        )

    def _request_test_web_link_document_type_remove_api_view(self):
        return self.post(
            viewname='rest_api:web_link-document_type-remove',
            kwargs={'web_link_id': self._test_web_link.pk}, data={
                'document_type': self._test_document_type.pk
            }
        )


class WebLinkDocumentTypeViewTestMixin(WebLinkTestMixin):
    def _request_test_web_link_document_type_add_remove_get_view(self):
        return self.get(
            viewname='web_links:web_link_document_types', kwargs={
                'web_link_id': self._test_web_link.pk
            }
        )

    def _request_test_web_link_document_type_add_view(self):
        return self.post(
            viewname='web_links:web_link_document_types', kwargs={
                'web_link_id': self._test_web_link.pk
            }, data={
                'available-selection': self._test_document_type.pk,
                'available-submit': 'true'
            }
        )

    def _request_test_web_link_document_type_remove_view(self):
        return self.post(
            viewname='web_links:web_link_document_types', kwargs={
                'web_link_id': self._test_web_link.pk
            }, data={
                'added-selection': self._test_document_type.pk,
                'added-submit': 'true'
            }
        )


class WebLinkViewTestMixin(WebLinkTestMixin):
    def _request_test_document_web_link_instance_view(self):
        return self.post(
            viewname='web_links:web_link_instance_view', kwargs={
                'document_id': self._test_document.pk,
                'web_link_id': self._test_web_link.pk
            }
        )

    def _request_test_document_web_link_list_view(self):
        return self.get(
            viewname='web_links:document_web_link_list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_web_link_create_view(self):
        self._test_object_track()

        response = self.post(
            viewname='web_links:web_link_create', data={
                'label': TEST_WEB_LINK_LABEL,
                'template_template': TEST_WEB_LINK_TEMPLATE
            }
        )

        self._test_object_set()

        return response

    def _request_test_web_link_delete_view(self):
        return self.post(
            viewname='web_links:web_link_delete', kwargs={
                'web_link_id': self._test_web_link.pk
            }
        )

    def _request_test_web_link_edit_view(self):
        return self.post(
            viewname='web_links:web_link_edit', kwargs={
                'web_link_id': self._test_web_link.pk
            }, data={
                'label': TEST_WEB_LINK_LABEL_EDITED,
                'template_template': TEST_WEB_LINK_TEMPLATE
            }
        )

    def _request_test_web_link_list_view(self):
        return self.get(viewname='web_links:web_link_list')



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('sources', '0025_delete_sourcelog')
    ]

    operations = [
        migrations.AddField(
            model_name='source', name='backend_data',
            field=models.TextField(blank=True, verbose_name='Backend data')
        ),
        migrations.AddField(
            model_name='source', name='backend_path',
            field=models.CharField(
                default='', help_text='The dotted Python path to the '
                'backend class.', max_length=128, verbose_name='Backend path'
            ),
            preserve_default=False
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='StoredCredential',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'label', models.CharField(
                        help_text='Short description of this credential.',
                        max_length=128, unique=True, verbose_name='Label'
                    )
                ),
                (
                    'backend_path', models.CharField(
                        help_text='The dotted Python path to the backend '
                        'class.', max_length=128, verbose_name='Backend path'
                    )
                ),
                (
                    'backend_data', models.TextField(
                        blank=True, verbose_name='Backend data'
                    )
                )
            ],
            options={
                'verbose_name': 'Credential',
                'verbose_name_plural': 'Credentials',
                'ordering': ('label',)
            }
        )
    ]



from rest_framework import status

from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import event_message_created, event_message_edited
from ..models import Message
from ..permissions import (
    permission_message_create, permission_message_delete,
    permission_message_edit, permission_message_view
)

from .mixins import MessageAPIViewTestMixin


class MessageAPIViewTestCase(MessageAPIViewTestMixin, BaseAPITestCase):
    auto_create_test_message = False

    def test_message_create_api_view_no_permission(self):
        message_count = Message.objects.count()

        self._clear_events()

        response = self._request_test_message_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertEqual(Message.objects.count(), message_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_create_api_view_with_permission(self):
        self.grant_permission(permission=permission_message_create)

        message_count = Message.objects.count()

        self._clear_events()

        response = self._request_test_message_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        self.assertEqual(Message.objects.count(), message_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_message)
        self.assertEqual(events[0].verb, event_message_created.id)

    def test_message_create_api_view_for_super_user_with_permission(self):
        self.grant_permission(permission=permission_message_create)

        message_count = Message.objects.count()

        self._create_test_super_user()

        self._clear_events()

        response = self._request_test_message_create_api_view(
            extra_data={'user': self._test_super_user.pk}
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(Message.objects.count(), message_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_delete_api_view_no_permission(self):
        self._create_test_message()

        message_count = Message.objects.count()

        self._clear_events()

        response = self._request_test_message_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(Message.objects.count(), message_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_delete_api_view_with_access(self):
        self._create_test_message()

        self.grant_access(
            obj=self._test_message, permission=permission_message_delete
        )

        message_count = Message.objects.count()

        self._clear_events()

        response = self._request_test_message_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(Message.objects.count(), message_count - 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_detail_api_view_no_permission(self):
        self._create_test_message()

        self._clear_events()

        response = self._request_test_message_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_detail_api_view_with_access(self):
        self._create_test_message()
        self.grant_access(
            obj=self._test_message, permission=permission_message_view
        )

        self._clear_events()

        response = self._request_test_message_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(response.data['id'], self._test_message.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_edit_api_view_via_patch_no_permission(self):
        self._create_test_message()

        message_read = self._test_message.read

        self._clear_events()

        response = self._request_test_message_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_message.refresh_from_db()
        self.assertEqual(self._test_message.read, message_read)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_edit_api_view_via_patch_with_access(self):
        self._create_test_message()

        self.grant_access(
            obj=self._test_message, permission=permission_message_edit
        )

        message_read = self._test_message.read

        self._clear_events()

        response = self._request_test_message_edit_api_view(verb='patch')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_message.refresh_from_db()
        self.assertNotEqual(self._test_message.read, message_read)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_message)
        self.assertEqual(events[0].verb, event_message_edited.id)

    def test_message_edit_api_view_via_put_no_permission(self):
        self._create_test_message()

        message_read = self._test_message.read

        self._clear_events()

        response = self._request_test_message_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_message.refresh_from_db()
        self.assertEqual(self._test_message.read, message_read)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_edit_api_view_via_put_with_access(self):
        self._create_test_message()

        self.grant_access(
            obj=self._test_message, permission=permission_message_edit
        )

        message_read = self._test_message.read

        self._clear_events()

        response = self._request_test_message_edit_api_view(verb='put')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_message.refresh_from_db()
        self.assertNotEqual(self._test_message.read, message_read)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_message)
        self.assertEqual(events[0].verb, event_message_edited.id)

    def test_message_list_api_view_no_permission(self):
        self._create_test_message()

        self._clear_events()

        response = self._request_test_message_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['count'], 0
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_message_list_api_view_with_access(self):
        self._create_test_message()
        self.grant_access(
            obj=self._test_message, permission=permission_message_view
        )

        self._clear_events()

        response = self._request_test_message_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['id'],
            self._test_message.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django.http import HttpResponseRedirect
from django.urls import reverse

from mayan.apps.common.settings import setting_home_view

from .utils import is_otp_backend_enabled


class OTPBackendEnabledViewMixin:
    def dispatch(self, request, *args, **kwargs):
        if not is_otp_backend_enabled():
            return HttpResponseRedirect(
                redirect_to=reverse(viewname=setting_home_view.value)
            )
        else:
            return super().dispatch(request=request, *args, **kwargs)



from django.apps import apps

from .exceptions import DocumentNotCheckedOut, NewDocumentFileNotAllowed


def hook_is_new_file_allowed(instance, kwargs=None):
    DocumentCheckout = apps.get_model(
        app_label='checkouts', model_name='DocumentCheckout'
    )

    document = kwargs.get('document', None)
    # Do not put "instance.document" as the default for .get() as it gets
    # evaluated before .get() is executed and returns an AttributeError when
    # instance is None.
    if not document:
        if not instance:
            return
        else:
            document = instance.document

    try:
        checkout_information = DocumentCheckout.objects.get_check_out_info(
            document=document
        )
    except DocumentNotCheckedOut:
        """Document is not checked out, do nothing"""
    else:
        if checkout_information.user != kwargs['user'] and checkout_information.block_new_file:
            raise NewDocumentFileNotAllowed



import os

from django.conf import settings
from django.utils.encoding import force_bytes

from mayan.apps.storage.utils import NamedTemporaryFile, fs_cleanup
from mayan.apps.testing.tests.mixins import EnvironmentTestCaseMixin

from ..classes import Setting
from ..settings import setting_cluster
from ..utils import BaseSetting, SettingNamespaceSingleton

from .literals import (
    TEST_BOOTSTAP_SETTING_NAME, TEST_NAMESPACE_LABEL, TEST_NAMESPACE_NAME,
    TEST_SETTING_DEFAULT_VALUE, TEST_SETTING_GLOBAL_NAME
)


class BoostrapSettingTestMixin:
    def _create_test_bootstrap_singleton(self):
        self.test_globals = {}
        self.test_globals['BASE_DIR'] = ''
        self._test_setting_namespace_singleton = SettingNamespaceSingleton(
            global_symbol_table=self.test_globals
        )

    def _register_test_boostrap_setting(self):
        SettingNamespaceSingleton.register_setting(
            name=TEST_BOOTSTAP_SETTING_NAME, klass=BaseSetting, kwargs={
                'has_default': True, 'default_value': 'value default'
            }
        )


class SettingClusterTestMixin(EnvironmentTestCaseMixin):
    def setUp(self):
        super().setUp()
        setting_cluster.do_cache_invalidate()

        with NamedTemporaryFile(delete=False) as self._test_setting_config_file_object:
            settings.CONFIGURATION_FILEPATH = self._test_setting_config_file_object.name
            os.environ['MAYAN_CONFIGURATION_FILEPATH'] = self._test_setting_config_file_object.name

        setting_cluster.do_cache_invalidate()

    def tearDown(self):
        fs_cleanup(filename=self._test_setting_config_file_object.name)
        setting_cluster.do_cache_invalidate()
        super().tearDown()


class SettingClusterViewTestMixin(SettingClusterTestMixin):
    def _request_cluster_configuration_save_view(self):
        return self.post(
            viewname='settings:setting_cluster_configuration_save'
        )


class SettingNamespaceTestMixin(SettingClusterTestMixin):
    _test_setting_namespace_create_auto = False

    def setUp(self):
        super().setUp()
        self._test_setting_namespace_list = []

        if self._test_setting_namespace_create_auto:
            self._test_setting_namespace_create()

    def tearDown(self):
        for test_setting_namespace in self._test_setting_namespace_list:
            try:
                setting_cluster.do_namespace_remove(
                    name=test_setting_namespace.name
                )
            except KeyError:
                """
                Test setting namespace was removed by the test itself.
                """

        super().tearDown()

    def _test_setting_namespace_create(self, name=None, label=None, **kwargs):
        test_setting_namespace_list_count = len(
            self._test_setting_namespace_list
        )

        if name is None:
            name = '{}_{}'.format(
                TEST_NAMESPACE_NAME, test_setting_namespace_list_count
            )

        if label is None:
            label = '{}_{}'.format(
                TEST_NAMESPACE_LABEL, test_setting_namespace_list_count
            )

        self._test_setting_namespace = setting_cluster.do_namespace_add(
            label=label, name=name, **kwargs
        )
        self._test_setting_namespace_list.append(
            self._test_setting_namespace
        )


class SettingNamespaceViewTestMixin(SettingNamespaceTestMixin):
    def _request_namespace_detail_view(self):
        return self.get(
            viewname='settings:setting_namespace_detail', kwargs={
                'namespace_name': self._test_setting_namespace.name
            }
        )

    def _request_namespace_list_view(self):
        return self.get(viewname='settings:setting_cluster_namespace_list')


class SettingTestMixin(SettingNamespaceTestMixin):
    _test_configuration_file_object = None
    _test_configuration_value = None
    _test_setting_global_name = None

    def tearDown(self):
        if self._test_configuration_file_object:
            fs_cleanup(filename=self._test_configuration_file_object.name)
        super().tearDown()

    def _create_test_configuration_file(self, callback=None):
        if not self._test_setting_global_name:
            self._test_setting_global_name = self._test_setting.global_name

        test_config_entry = {
            self._test_setting_global_name: self._test_configuration_value
        }

        with NamedTemporaryFile(delete=False) as _test_configuration_file_object:
            # Needed to load the config file from the Setting class
            # after bootstrap.
            settings.CONFIGURATION_FILEPATH = _test_configuration_file_object.name
            # Needed to update the globals before Mayan has loaded.
            self._set_environment_variable(
                name='MAYAN_CONFIGURATION_FILEPATH',
                value=_test_configuration_file_object.name
            )
            _test_configuration_file_object.write(
                force_bytes(
                    s=Setting.serialize_value(value=test_config_entry)
                )
            )
            _test_configuration_file_object.seek(0)
            setting_cluster.configuration_file_cache = None

            if callback:
                callback()

    def _create_test_setting(self, validation_function=None):
        self._test_setting = self._test_setting_namespace.do_setting_add(
            default=TEST_SETTING_DEFAULT_VALUE,
            global_name=TEST_SETTING_GLOBAL_NAME,
            validation_function=validation_function
        )


class SettingViewTestMixin(SettingTestMixin):
    def _request_setting_edit_view(self, value):
        return self.post(
            viewname='settings:setting_edit_view', kwargs={
                'setting_global_name': self._test_setting.global_name
            }, data={'value': value}
        )

    def _request_setting_revert_view(self):
        return self.post(
            viewname='settings:setting_revert_view', kwargs={
                'setting_global_name': self._test_setting.global_name
            }
        )



import datetime
from errno import ENOENT
import logging
from stat import S_IFDIR, S_IFREG
from time import time

from fuse import FuseOSError, LoggingMixIn, Operations

from django.core.exceptions import MultipleObjectsReturned
from django.db.models import (
    Case, CharField, Count, F, Func, Transform, Value, When
)
from django.db.models.functions import Concat

from mayan.apps.documents.models.document_models import Document

from .literals import (
    DIRECTORY_MODE, FILE_MODE, MAX_FILE_DESCRIPTOR, MIN_FILE_DESCRIPTOR
)
from .runtime import cache

logger = logging.getLogger(name=__name__)


class Trim(Transform):
    function = 'TRIM'
    lookup_name = 'trim'


class MirrorFilesystem(LoggingMixIn, Operations):
    @staticmethod
    def _clean_queryset(queryset, source_field_name, destination_field_name):
        queryset = MirrorFilesystem._clean_queryset_end_of_lines(
            queryset=queryset, source_field_name=source_field_name,
            destination_field_name='_no_newline'
        )

        queryset = MirrorFilesystem._clean_queryset_slashes(
            queryset=queryset, source_field_name='_no_newline'
        )

        return MirrorFilesystem._clean_queryset_duplicates(
            queryset=queryset, destination_field_name=destination_field_name
        )

    @staticmethod
    def _clean_queryset_end_of_lines(
        queryset, source_field_name, destination_field_name='clean_value'
    ):
        # Remove newline carriage returns and the first and last space
        # to make multiline indexes valid directory names.
        return queryset.annotate(
            **{
                destination_field_name: Trim(
                    Func(
                        F(source_field_name), Value('\r\n'), Value(' '),
                        function='replace', output_field=CharField()
                    )
                )
            }
        )

    @staticmethod
    def _clean_queryset_slashes(
        queryset, source_field_name, destination_field_name='_no_slashes'
    ):
        # This is a conditional expression that is executed only for
        # items in the queryset that contain a slash ('/') in their source
        # field. The slash ('/') character is replaced with an
        # underscore ('_').
        return queryset.annotate(
            **{
                destination_field_name: Case(
                    When(
                        **{
                            '{}__contains'.format(source_field_name): '/',
                            'then': Func(
                                F(source_field_name), Value('/'), Value('_'),
                                function='replace', output_field=CharField()
                            )
                        }
                    ),
                    default=source_field_name
                )
            }
        )

    @staticmethod
    def _clean_queryset_duplicates(
        queryset, destination_field_name='_deduplicated',
        source_field_name='_no_slashes'
    ):
        # Make second queryset of all duplicates.
        repeats = queryset.values(source_field_name).annotate(
            repeated_count=Count(source_field_name)
        ).filter(repeated_count__gt=1).values(source_field_name)

        # This is a conditional expression that is executed only for
        # duplicates. The primary key is appended inside a parenthesis to
        # the source field.
        return queryset.annotate(
            **{
                destination_field_name: Case(
                    When(
                        **{
                            '{}__in'.format(source_field_name): repeats,
                            'then': Concat(
                                F(source_field_name), Value('('), F('pk'),
                                Value(')'), output_field=CharField()
                            )
                        }
                    ),
                    default=source_field_name
                )
            }
        )

    def _get_next_file_descriptor(self):
        while (True):
            self.file_descriptor_count += 1
            if self.file_descriptor_count > MAX_FILE_DESCRIPTOR:
                self.file_descriptor_count = MIN_FILE_DESCRIPTOR

            try:
                if not self.file_descriptors[self.file_descriptor_count]:
                    return self.file_descriptor_count
            except KeyError:
                return self.file_descriptor_count

    def _path_to_node(self, path, access_only=False, directory_only=True):
        logger.debug('path: %s', path)
        logger.debug('directory_only: %s', directory_only)

        parts = path.split('/')

        logger.debug('parts: %s', parts)

        node = self.func_document_container_node()

        if len(parts) > 1 and parts[1] != '':
            path_cache = cache.get_path(path=path)

            if path_cache:
                node_pk = path_cache.get('node_pk')
                if node_pk:
                    if access_only:
                        return True
                    else:
                        return self.func_document_container_node().get_descendants(
                            include_self=True
                        ).get(pk=node_pk)

                document_pk = path_cache.get('document_pk')
                if document_pk:
                    if access_only:
                        return True
                    else:
                        return Document.valid.get(pk=document_pk)

            for count, part in enumerate(iterable=parts[1:]):
                try:
                    queryset_nodes = MirrorFilesystem._clean_queryset(
                        queryset=node.get_descendants(include_self=True),
                        destination_field_name='value_clean',
                        source_field_name=self.node_text_attribute
                    )
                    node = queryset_nodes.get(value_clean=part)
                except self.func_document_container_node()._meta.model.DoesNotExist:
                    logger.debug('%s does not exists', part)

                    if directory_only:
                        return None
                    else:
                        try:
                            document = MirrorFilesystem._clean_queryset(
                                destination_field_name='label_clean',
                                queryset=node._get_documents(),
                                source_field_name='label'
                            ).get(label_clean=part)

                            logger.debug(
                                'path %s is a valid file path', path
                            )
                            cache.set_path(path=path, document=document)

                            return document
                        except Document.DoesNotExist:
                            logger.debug(
                                'path %s is a file, but is not found', path
                            )
                            return None
                        except MultipleObjectsReturned:
                            return None
                except MultipleObjectsReturned:
                    return None

            cache.set_path(path=path, node=node)

        logger.debug('node: %s', node)
        logger.debug(
            'node is root: %s', node.is_root_node()
        )

        return node

    def __init__(self, func_document_container_node, node_text_attribute):
        self.file_descriptor_count = MIN_FILE_DESCRIPTOR
        self.file_descriptors = {}
        self.func_document_container_node = func_document_container_node
        self.node_text_attribute = node_text_attribute

    def access(self, path, fh=None):
        result = self._path_to_node(
            directory_only=False, path=path, access_only=True
        )

        if not result:
            raise FuseOSError(ENOENT)

    def getattr(self, path, fh=None):
        logger.debug('path: %s, fh: %s', path, fh)

        now = time()
        result = self._path_to_node(path=path, directory_only=False)

        if not result:
            raise FuseOSError(ENOENT)

        # st_nlink tracks the number of hard links to a file.
        # Must be 2 for directories and at least 1 for files.
        # https://www.gnu.org/software/libc/manual/html_node/Attribute-Meanings.html
        if isinstance(result, Document):
            function_result = {
                'st_mode': (S_IFREG | FILE_MODE),
                'st_ctime': (
                    result.datetime_created.replace(
                        tzinfo=None
                    ) - result.datetime_created.utcoffset() - datetime.datetime(1970, 1, 1)
                ).total_seconds(),
                'st_mtime': (
                    result.file_latest.timestamp.replace(
                        tzinfo=None
                    ) - result.file_latest.timestamp.utcoffset() - datetime.datetime(1970, 1, 1)
                ).total_seconds(),
                'st_atime': now,
                'st_size': result.file_latest.size or 0,
                'st_nlink': 1
            }
        else:
            function_result = {
                'st_atime': now, 'st_ctime': now,
                'st_mode': (S_IFDIR | DIRECTORY_MODE), 'st_mtime': now,
                'st_nlink': 2
            }

        logger.debug('function_result: %s', function_result)
        return function_result

    def open(self, path, flags):
        result = self._path_to_node(path=path, directory_only=False)

        if isinstance(result, Document):
            next_file_descriptor = self._get_next_file_descriptor()
            self.file_descriptors[next_file_descriptor] = result.file_latest.open()
            return next_file_descriptor
        else:
            raise FuseOSError(ENOENT)

    def read(self, path, size, offset, fh):
        self.file_descriptors[fh].seek(offset)
        return self.file_descriptors[fh].read(size)

    def readdir(self, path, fh):
        logger.debug('path: %s', path)

        node = self._path_to_node(path=path, directory_only=True)

        if not node:
            raise FuseOSError(ENOENT)

        yield '.'
        yield '..'

        # Serve nodes as directories.
        queryset = MirrorFilesystem._clean_queryset(
            queryset=node.get_children(),
            source_field_name=self.node_text_attribute,
            destination_field_name='value_clean'
        )

        for value in queryset.values_list('value_clean', flat=True):
            yield value

        # Then serve nodes documents as files.
        queryset = MirrorFilesystem._clean_queryset(
            destination_field_name='label_clean',
            queryset=node._get_documents(), source_field_name='label'
        )

        for value in queryset.values_list('label_clean', flat=True):
            yield value

    def release(self, path, fh):
        self.file_descriptors[fh] = None
        del (
            self.file_descriptors[fh]
        )



from mayan.apps.documents.literals import DEFAULT_DOCUMENT_FILE_ACTION_NAME
from mayan.apps.documents.tests.literals import TEST_FILE_SMALL_PATH
from mayan.apps.source_compressed.source_backends.literals import SOURCE_UNCOMPRESS_CHOICE_NEVER
from mayan.apps.sources.tests.literals import (
    TEST_CASE_ACTION_NAME_SOURCE_CREATE, TEST_CASE_INTERFACE_NAME_MODEL,
    TEST_CASE_INTERFACE_NAME_REST_API, TEST_CASE_INTERFACE_NAME_VIEW
)
from mayan.apps.sources.tests.mixins.base_mixins import SourceTestMixin

from .literals import TEST_SOURCE_BACKEND_PATH_WEB_FORM


class WebFormSourceTestMixin(SourceTestMixin):
    _test_source_backend_path = TEST_SOURCE_BACKEND_PATH_WEB_FORM
    _test_source_file_path = TEST_FILE_SMALL_PATH

    def _get_test_source_backend_data(self, action_name, interface_name):
        result = super()._get_test_source_backend_data(
            action_name=action_name, interface_name=interface_name
        )

        if action_name == TEST_CASE_ACTION_NAME_SOURCE_CREATE:
            result['uncompress'] = SOURCE_UNCOMPRESS_CHOICE_NEVER
        elif action_name == 'document_file_upload':
            if interface_name == TEST_CASE_INTERFACE_NAME_MODEL:
                result['document'] = self._test_document
            elif interface_name == TEST_CASE_INTERFACE_NAME_REST_API:
                result['document_id'] = self._test_document.pk
            elif interface_name == TEST_CASE_INTERFACE_NAME_VIEW:
                result['document-action_name'] = DEFAULT_DOCUMENT_FILE_ACTION_NAME
        elif action_name == 'document_upload':
            if interface_name == TEST_CASE_INTERFACE_NAME_MODEL:
                result['document_type'] = self._test_document_type
            elif interface_name == TEST_CASE_INTERFACE_NAME_REST_API:
                result['document_type_id'] = self._test_document_type.pk

        return result



from django.utils.translation import gettext_lazy as _

from mayan.apps.sources.source_backend_actions.interface_arguments import SourceBackendActionInterfaceArgument

argument_dry_run = SourceBackendActionInterfaceArgument(
    default=False, help_text=_(
        'Executes the primary action of the source in test mode. '
        'Permanent modifications like deletion of files are disabled.'
    ), required=False
)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0089_auto_20230829_0819')
    ]

    operations = [
        migrations.AlterField(
            model_name='documentversion', name='active',
            field=models.BooleanField(
                default=False, help_text='Determines the active version '
                'of the document.', verbose_name='Active'
            )
        )
    ]



from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from django.utils.translation import gettext_lazy as _


class DocumentFileDriverEntryBusinessLogicMixin:
    def get_attribute_count(self):
        return self.entries.count()
    get_attribute_count.short_description = _(message='Attribute count')


class StoredDriverBusinessLogicMixin:
    @cached_property
    def driver_class(self):
        return import_string(dotted_path=self.driver_path)

    @cached_property
    def driver_label(self):
        return self.driver_class.label



import json

from django import forms
from django.db import models
from django.utils.translation import gettext_lazy as _

from mayan.apps.views.forms import DynamicModelForm

from .classes import QuotaBackend
from .models import Quota


class QuotaBackendSelectionForm(forms.Form):
    backend = forms.ChoiceField(
        choices=(), label=_(message='Backend'), help_text=_(
            'The quota driver for this entry.'
        )
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['backend'].choices = QuotaBackend.get_choices()


class QuotaDynamicForm(DynamicModelForm):
    class Meta:
        fields = ('enabled', 'backend_data')
        model = Quota
        widgets = {'backend_data': forms.widgets.HiddenInput}

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop(
            'user', {}
        )
        result = super().__init__(*args, **kwargs)

        # Handle filtered queryset fields.
        for field in self.fields:
            if hasattr(self.fields[field], 'reload'):
                self.fields[field].user = self.user
                self.fields[field].reload()

        if self.instance.backend_data:
            for key, value in json.loads(s=self.instance.backend_data).items():
                self.fields[key].initial = value

        return result

    def clean(self):
        data = super().clean()
        # Consolidate the dynamic fields into a single JSON field called
        # 'backend_data'.
        backend_data = {}

        for field_name, field_data in self.schema['fields'].items():
            field_data = data.pop(
                field_name, field_data.get('default', None)
            )

            # Reduce models to a pk.
            if isinstance(field_data, models.Model):
                field_data = field_data.pk

            # Reduce querysets to a list.
            if isinstance(field_data, models.query.QuerySet):
                field_data = list(
                    field_data.values_list('pk', flat=True)
                )

            backend_data[field_name] = field_data

        data['backend_data'] = json.dumps(obj=backend_data)
        return data



from django.db import models
from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import EventManagerSave

from .events import event_quota_created, event_quota_edited
from .model_mixins import QuotaBusinessModelMixin


class Quota(ExtraDataModelMixin, QuotaBusinessModelMixin, models.Model):
    backend_path = models.CharField(
        max_length=255, help_text=_(
            'The dotted Python path to the backend class.'
        ), verbose_name=_(message='Backend path')
    )
    backend_data = models.TextField(
        blank=True, verbose_name=_(message='Backend data')
    )
    enabled = models.BooleanField(
        default=True, help_text=_(
            'Allow quick disable or enable of the quota.'
        ), verbose_name=_(message='Enabled')
    )

    class Meta:
        ordering = ('id',)
        verbose_name = _(message='Quota')
        verbose_name_plural = _(message='Quotas')

    def __str__(self):
        return str(
            self.backend_label()
        )

    def get_absolute_url(self):
        return reverse(viewname='quotas:quota_list')

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'event': event_quota_created,
            'target': 'self'
        },
        edited={
            'event': event_quota_edited,
            'target': 'self'
        }
    )
    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)



TEST_DRIVER_CLASS_PATH = 'mayan.apps.file_metadata.tests.drivers.FileMetadataDriverTest'
TEST_DRIVER_INTERNAL_NAME = 'test_driver'
TEST_DRIVER_LABEL = 'test label'

TEST_FILE_METADATA_KEY = 'test_key'
TEST_FILE_METADATA_VALUE = 'test_value'

TEST_FILE_METADATA_INDEX_NODE_TEMPLATE = "{{{{ document.file_metadata_value_of.{}__{} }}}}".format(
    TEST_DRIVER_INTERNAL_NAME, TEST_FILE_METADATA_KEY
)

# MSG

TEST_MSG_FILE_METADATA_DOTTED_NAME_SUBJECT = 'extract_msg__subject'
TEST_MSG_FILE_METADATA_DOTTED_NAME_TO = 'extract_msg__to'

TEST_MSG_FILE_METADATA_VALUE_SUBJECT = 'MSG Test File'
TEST_MSG_FILE_METADATA_VALUE_TO = 'time2talk@online-convert.com <time2talk@online-convert.com>'

# PDF

TEST_PDF_FILE_METADATA_DOTTED_NAME = 'exiftool__producer'
TEST_PDF_FILE_METADATA_VALUE = 'pdfTeX-1.40.10'



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('sources', '0027_auto_20201030_0259')
    ]

    operations = [
        migrations.RemoveField(
            model_name='imapemail',
            name='emailbasemodel_ptr'
        ),
        migrations.RemoveField(
            model_name='pop3email',
            name='emailbasemodel_ptr'
        ),
        migrations.RemoveField(
            model_name='emailbasemodel',
            name='intervalbasemodel_ptr'
        ),
        migrations.RemoveField(
            model_name='sanescanner',
            name='interactivesource_ptr'
        ),
        migrations.RemoveField(
            model_name='stagingfoldersource',
            name='interactivesource_ptr'
        ),
        migrations.RemoveField(
            model_name='watchfoldersource',
            name='intervalbasemodel_ptr'
        ),
        migrations.RemoveField(
            model_name='webformsource',
            name='interactivesource_ptr'
        ),
        migrations.RemoveField(
            model_name='intervalbasemodel',
            name='document_type'
        ),
        migrations.RemoveField(
            model_name='intervalbasemodel',
            name='outofprocesssource_ptr'
        ),
        migrations.DeleteModel(
            name='EmailBaseModel'
        ),
        migrations.DeleteModel(
            name='IMAPEmail'
        ),
        migrations.DeleteModel(
            name='InteractiveSource'
        ),
        migrations.DeleteModel(
            name='IntervalBaseModel'
        ),
        migrations.DeleteModel(
            name='OutOfProcessSource'
        ),
        migrations.DeleteModel(
            name='POP3Email'
        ),
        migrations.DeleteModel(
            name='SaneScanner'
        ),
        migrations.DeleteModel(
            name='StagingFolderSource'
        ),
        migrations.DeleteModel(
            name='WatchFolderSource'
        ),
        migrations.DeleteModel(
            name='WebFormSource'
        ),
        migrations.AlterField(
            model_name='source',
            name='backend_data',
            field=models.TextField(
                blank=True, help_text='JSON encoded data for the backend '
                'class.', verbose_name='Backend data'
            )
        )
    ]



# -*- coding: utf-8 -*-

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0025_auto_20150718_0742'),
    ]

    operations = [
        migrations.AlterField(
            model_name='document',
            name='language',
            field=models.CharField(
                default='eng', max_length=8, verbose_name='Language',
                blank=True, choices=[
                    ['aar', 'Afar'], ['abk', 'Abkhazian'],
                    ['ace', 'Achinese'], ['ach', 'Acoli'],
                    ['ada', 'Adangme'], ['ady', 'Adyghe; Adygei'],
                    ['afa', 'Afro-Asiatic languages'], ['afh', 'Afrihili'],
                    ['afr', 'Afrikaans'], ['ain', 'Ainu'],
                    ['aka', 'Akan'], ['akk', 'Akkadian'],
                    ['sqi', 'Albanian'],
                    ['ale', 'Aleut'], ['alg', 'Algonquian languages'],
                    ['alt', 'Southern Altai'], ['amh', 'Amharic'],
                    ['ang', 'English, Old (ca. 450-1100)'],
                    ['anp', 'Angika'], ['apa', 'Apache languages'],
                    ['ara', 'Arabic'],
                    [
                        b'arc', 'Official Aramaic (700-300 BCE); Imperial '
                        'Aramaic (700-300 BCE)'
                    ], ['arg', 'Aragonese'], ['hye', 'Armenian'],
                    ['arn', 'Mapudungun; Mapuche'], ['arp', 'Arapaho'],
                    ['art', 'Artificial languages'], ['arw', 'Arawak'],
                    ['asm', 'Assamese'],
                    ['ast', 'Asturian; Bable; Leonese; Asturleonese'],
                    ['ath', 'Athapascan languages'],
                    ['aus', 'Australian languages'],
                    ['ava', 'Avaric'], ['ave', 'Avestan'],
                    ['awa', 'Awadhi'], ['aym', 'Aymara'],
                    ['aze', 'Azerbaijani'], ['bad', 'Banda languages'],
                    ['bai', 'Bamileke languages'], ['bak', 'Bashkir'],
                    ['bal', 'Baluchi'], ['bam', 'Bambara'],
                    ['ban', 'Balinese'], ['eus', 'Basque'],
                    ['bas', 'Basa'], ['bat', 'Baltic languages'],
                    ['bej', 'Beja; Bedawiyet'], ['bel', 'Belarusian'],
                    ['bem', 'Bemba'], ['ben', 'Bengali'],
                    ['ber', 'Berber languages'], ['bho', 'Bhojpuri'],
                    ['bih', 'Bihari languages'], ['bik', 'Bikol'],
                    ['bin', 'Bini; Edo'], ['bis', 'Bislama'],
                    ['bla', 'Siksika'], ['bnt', 'Bantu languages'],
                    ['bos', 'Bosnian'], ['bra', 'Braj'],
                    ['bre', 'Breton'], ['btk', 'Batak languages'],
                    ['bua', 'Buriat'], ['bug', 'Buginese'],
                    ['bul', 'Bulgarian'], ['mya', 'Burmese'],
                    ['byn', 'Blin; Bilin'], ['cad', 'Caddo'],
                    ['cai', 'Central American Indian languages'],
                    ['car', 'Galibi Carib'], ['cat', 'Catalan; Valencian'],
                    ['cau', 'Caucasian languages'], ['ceb', 'Cebuano'],
                    ['cel', 'Celtic languages'], ['cha', 'Chamorro'],
                    ['chb', 'Chibcha'], ['che', 'Chechen'],
                    ['chg', 'Chagatai'], ['zho', 'Chinese'],
                    ['chk', 'Chuukese'], ['chm', 'Mari'],
                    ['chn', 'Chinook jargon'], ['cho', 'Choctaw'],
                    ['chp', 'Chipewyan; Dene Suline'], ['chr', 'Cherokee'],
                    [
                        b'chu', 'Church Slavic; Old Slavonic; Church '
                        'Slavonic; Old Bulgarian; Old Church Slavonic'
                    ],
                    ['chv', 'Chuvash'], ['chy', 'Cheyenne'],
                    ['cmc', 'Chamic languages'], ['cop', 'Coptic'],
                    ['cor', 'Cornish'], ['cos', 'Corsican'],
                    ['cpe', 'Creoles and pidgins, English based'],
                    ['cpf', 'Creoles and pidgins, French-based'],
                    ['cpp', 'Creoles and pidgins, Portuguese-based'],
                    ['cre', 'Cree'],
                    ['crh', 'Crimean Tatar; Crimean Turkish'],
                    ['crp', 'Creoles and pidgins'], ['csb', 'Kashubian'],
                    ['cus', 'Cushitic languages'], ['ces', 'Czech'],
                    ['dak', 'Dakota'], ['dan', 'Danish'],
                    ['dar', 'Dargwa'], ['day', 'Land Dayak languages'],
                    ['del', 'Delaware'], ['den', 'Slave (Athapascan)'],
                    ['dgr', 'Dogrib'], ['din', 'Dinka'],
                    ['div', 'Divehi; Dhivehi; Maldivian'],
                    ['doi', 'Dogri'], ['dra', 'Dravidian languages'],
                    ['dsb', 'Lower Sorbian'], ['dua', 'Duala'],
                    ['dum', 'Dutch, Middle (ca. 1050-1350)'],
                    ['nld', 'Dutch; Flemish'], ['dyu', 'Dyula'],
                    ['dzo', 'Dzongkha'], ['efi', 'Efik'],
                    ['egy', 'Egyptian (Ancient)'], ['eka', 'Ekajuk'],
                    ['elx', 'Elamite'], ['eng', 'English'],
                    ['enm', 'English, Middle (1100-1500)'],
                    ['epo', 'Esperanto'], ['est', 'Estonian'],
                    ['ewe', 'Ewe'], ['ewo', 'Ewondo'], ['fan', 'Fang'],
                    ['fao', 'Faroese'], ['fat', 'Fanti'],
                    ['fij', 'Fijian'], ['fil', 'Filipino; Pilipino'],
                    ['fin', 'Finnish'], ['fiu', 'Finno-Ugrian languages'],
                    ['fon', 'Fon'], ['fra', 'French'],
                    ['frm', 'French, Middle (ca. 1400-1600)'],
                    ['fro', 'French, Old (842-ca. 1400)'],
                    ['frr', 'Northern Frisian'],
                    ['frs', 'Eastern Frisian'],
                    ['fry', 'Western Frisian'],
                    ['ful', 'Fulah'], ['fur', 'Friulian'], ['gaa', 'Ga'],
                    ['gay', 'Gayo'], ['gba', 'Gbaya'],
                    ['gem', 'Germanic languages'], ['kat', 'Georgian'],
                    ['deu', 'German'], ['gez', 'Geez'],
                    ['gil', 'Gilbertese'],
                    ['gla', 'Gaelic; Scottish Gaelic'], ['gle', 'Irish'],
                    ['glg', 'Galician'], ['glv', 'Manx'],
                    ['gmh', 'German, Middle High (ca. 1050-1500)'],
                    ['goh', 'German, Old High (ca. 750-1050)'],
                    ['gon', 'Gondi'], ['gor', 'Gorontalo'],
                    ['got', 'Gothic'], ['grb', 'Grebo'],
                    ['grc', 'Greek, Ancient (to 1453)'],
                    ['ell', 'Greek, Modern (1453-)'],
                    ['grn', 'Guarani'],
                    ['gsw', 'Swiss German; Alemannic; Alsatian'],
                    ['guj', 'Gujarati'], ['gwi', b"Gwich'in"],
                    ['hai', 'Haida'], ['hat', 'Haitian; Haitian Creole'],
                    ['hau', 'Hausa'], ['haw', 'Hawaiian'],
                    ['heb', 'Hebrew'], ['her', 'Herero'],
                    ['hil', 'Hiligaynon'],
                    ['him', 'Himachali languages; Western Pahari languages'],
                    ['hin', 'Hindi'], ['hit', 'Hittite'],
                    ['hmn', 'Hmong; Mong'], ['hmo', 'Hiri Motu'],
                    ['hrv', 'Croatian'], ['hsb', 'Upper Sorbian'],
                    ['hun', 'Hungarian'], ['hup', 'Hupa'],
                    ['iba', 'Iban'], ['ibo', 'Igbo'],
                    ['isl', 'Icelandic'], ['ido', 'Ido'],
                    ['iii', 'Sichuan Yi; Nuosu'],
                    ['ijo', 'Ijo languages'], ['iku', 'Inuktitut'],
                    ['ile', 'Interlingue; Occidental'], ['ilo', 'Iloko'],
                    [
                        b'ina', 'Interlingua (International Auxiliary '
                        'Language Association)'
                    ],
                    ['inc', 'Indic languages'], ['ind', 'Indonesian'],
                    ['ine', 'Indo-European languages'],
                    ['inh', 'Ingush'], ['ipk', 'Inupiaq'],
                    ['ira', 'Iranian languages'],
                    ['iro', 'Iroquoian languages'], ['ita', 'Italian'],
                    ['jav', 'Javanese'], ['jbo', 'Lojban'],
                    ['jpn', 'Japanese'], ['jpr', 'Judeo-Persian'],
                    ['jrb', 'Judeo-Arabic'], ['kaa', 'Kara-Kalpak'],
                    ['kab', 'Kabyle'], ['kac', 'Kachin; Jingpho'],
                    ['kal', 'Kalaallisut; Greenlandic'],
                    ['kam', 'Kamba'], ['kan', 'Kannada'],
                    ['kar', 'Karen languages'], ['kas', 'Kashmiri'],
                    ['kau', 'Kanuri'], ['kaw', 'Kawi'],
                    ['kaz', 'Kazakh'], ['kbd', 'Kabardian'],
                    ['kha', 'Khasi'], ['khi', 'Khoisan languages'],
                    ['khm', 'Central Khmer'], ['kho', 'Khotanese;Sakan'],
                    ['kik', 'Kikuyu; Gikuyu'], ['kin', 'Kinyarwanda'],
                    ['kir', 'Kirghiz; Kyrgyz'], ['kmb', 'Kimbundu'],
                    ['kok', 'Konkani'], ['kom', 'Komi'],
                    ['kon', 'Kongo'], ['kor', 'Korean'],
                    ['kos', 'Kosraean'], ['kpe', 'Kpelle'],
                    ['krc', 'Karachay-Balkar'], ['krl', 'Karelian'],
                    ['kro', 'Kru languages'], ['kru', 'Kurukh'],
                    ['kua', 'Kuanyama; Kwanyama'],
                    ['kum', 'Kumyk'], ['kur', 'Kurdish'],
                    ['kut', 'Kutenai'], ['lad', 'Ladino'],
                    ['lah', 'Lahnda'], ['lam', 'Lamba'],
                    ['lao', 'Lao'], ['lat', 'Latin'],
                    ['lav', 'Latvian'], ['lez', 'Lezghian'],
                    ['lim', 'Limburgan; Limburger; Limburgish'],
                    ['lin', 'Lingala'], ['lit', 'Lithuanian'],
                    ['lol', 'Mongo'], ['loz', 'Lozi'],
                    ['ltz', 'Luxembourgish; Letzeburgesch'],
                    ['lua', 'Luba-Lulua'], ['lub', 'Luba-Katanga'],
                    ['lug', 'Ganda'], ['lui', 'Luiseno'],
                    ['lun', 'Lunda'], ['luo', 'Luo (Kenya and Tanzania)'],
                    ['lus', 'Lushai'], ['mkd', 'Macedonian'],
                    ['mad', 'Madurese'], ['mag', 'Magahi'],
                    ['mah', 'Marshallese'], ['mai', 'Maithili'],
                    ['mak', 'Makasar'], ['mal', 'Malayalam'],
                    ['man', 'Mandingo'], ['mri', 'Maori'],
                    ['map', 'Austronesian languages'],
                    ['mar', 'Marathi'], ['mas', 'Masai'],
                    ['msa', 'Malay'], ['mdf', 'Moksha'],
                    ['mdr', 'Mandar'], ['men', 'Mende'],
                    ['mga', 'Irish, Middle (900-1200)'],
                    ['mic', b"Mi'kmaq; Micmac"], ['min', 'Minangkabau'],
                    ['mis', 'Uncoded languages'],
                    ['mkh', 'Mon-Khmer languages'],
                    ['mlg', 'Malagasy'], ['mlt', 'Maltese'],
                    ['mnc', 'Manchu'], ['mni', 'Manipuri'],
                    ['mno', 'Manobo languages'], ['moh', 'Mohawk'],
                    ['mol', 'Moldavian; Moldovan'],
                    ['mon', 'Mongolian'], ['mos', 'Mossi'],
                    ['mul', 'Multiple languages'],
                    ['mun', 'Munda languages'], ['mus', 'Creek'],
                    ['mwl', 'Mirandese'], ['mwr', 'Marwari'],
                    ['myn', 'Mayan languages'], ['myv', 'Erzya'],
                    ['nah', 'Nahuatl languages'],
                    ['nai', 'North American Indian languages'],
                    ['nap', 'Neapolitan'], ['nau', 'Nauru'],
                    ['nav', 'Navajo; Navaho'],
                    ['nbl', 'Ndebele, South; South Ndebele'],
                    ['nde', 'Ndebele, North; North Ndebele'],
                    ['ndo', 'Ndonga'],
                    [
                        b'nds', 'Low German; Low Saxon; German, Low; '
                        'Saxon, Low'
                    ],
                    ['nep', 'Nepali'], ['new', 'Nepal Bhasa; Newari'],
                    ['nia', 'Nias'],
                    ['nic', 'Niger-Kordofanian languages'],
                    ['niu', 'Niuean'],
                    ['nno', 'Norwegian Nynorsk; Nynorsk, Norwegian'],
                    ['nob', 'Bokm\xe5l, Norwegian; Norwegian Bokm\xe5l'],
                    ['nog', 'Nogai'], ['non', 'Norse, Old'],
                    ['nor', 'Norwegian'], ['nqo', b"N'Ko"],
                    ['nso', 'Pedi; Sepedi; Northern Sotho'],
                    ['nub', 'Nubian languages'],
                    [
                        b'nwc', 'Classical Newari; Old Newari; Classical '
                        'Nepal Bhasa'
                    ],
                    ['nya', 'Chichewa; Chewa; Nyanja'],
                    ['nym', 'Nyamwezi'], ['nyn', 'Nyankole'],
                    ['nyo', 'Nyoro'], ['nzi', 'Nzima'],
                    ['oci', 'Occitan (post 1500)'],
                    ['oji', 'Ojibwa'], ['ori', 'Oriya'],
                    ['orm', 'Oromo'], ['osa', 'Osage'],
                    ['oss', 'Ossetian; Ossetic'],
                    ['ota', 'Turkish, Ottoman (1500-1928)'],
                    ['oto', 'Otomian languages'],
                    ['paa', 'Papuan languages'],
                    ['pag', 'Pangasinan'], ['pal', 'Pahlavi'],
                    ['pam', 'Pampanga; Kapampangan'],
                    ['pan', 'Panjabi; Punjabi'],
                    ['pap', 'Papiamento'], ['pau', 'Palauan'],
                    ['peo', 'Persian, Old (ca. 600-400 B.C.)'],
                    ['fas', 'Persian'], ['phi', 'Philippine languages'],
                    ['phn', 'Phoenician'], ['pli', 'Pali'],
                    ['pol', 'Polish'], ['pon', 'Pohnpeian'],
                    ['por', 'Portuguese'], ['pra', 'Prakrit languages'],
                    [
                        b'pro', 'Proven\xe7al, Old (to 1500); Occitan, Old '
                        '(to 1500)'
                    ],
                    ['pus', 'Pushto; Pashto'],
                    ['qaa-qtz', 'Reserved for local use'],
                    ['que', 'Quechua'], ['raj', 'Rajasthani'],
                    ['rap', 'Rapanui'],
                    ['rar', 'Rarotongan; Cook Islands Maori'],
                    ['roa', 'Romance languages'], ['roh', 'Romansh'],
                    ['rom', 'Romany'], ['ron', 'Romanian'],
                    ['run', 'Rundi'],
                    ['rup', 'Aromanian; Arumanian; Macedo-Romanian'],
                    ['rus', 'Russian'], ['sad', 'Sandawe'],
                    ['sag', 'Sango'], ['sah', 'Yakut'],
                    ['sai', 'South American Indian languages'],
                    ['sal', 'Salishan languages'],
                    ['sam', 'Samaritan Aramaic'],
                    ['san', 'Sanskrit'], ['sas', 'Sasak'],
                    ['sat', 'Santali'], ['scn', 'Sicilian'],
                    ['sco', 'Scots'], ['sel', 'Selkup'],
                    ['sem', 'Semitic languages'],
                    ['sga', 'Irish, Old (to 900)'],
                    ['sgn', 'Sign Languages'],
                    ['shn', 'Shan'], ['sid', 'Sidamo'],
                    ['sin', 'Sinhala; Sinhalese'],
                    ['sio', 'Siouan languages'],
                    ['sit', 'Sino-Tibetan languages'],
                    ['sla', 'Slavic languages'],
                    ['slk', 'Slovak'], ['slv', 'Slovenian'],
                    ['sma', 'Southern Sami'], ['sme', 'Northern Sami'],
                    ['smi', 'Sami languages'], ['smj', 'Lule Sami'],
                    ['smn', 'Inari Sami'], ['smo', 'Samoan'],
                    ['sms', 'Skolt Sami'], ['sna', 'Shona'],
                    ['snd', 'Sindhi'], ['snk', 'Soninke'],
                    ['sog', 'Sogdian'], ['som', 'Somali'],
                    ['son', 'Songhai languages'],
                    ['sot', 'Sotho, Southern'],
                    ['spa', 'Spanish; Castilian'], ['srd', 'Sardinian'],
                    ['srn', 'Sranan Tongo'], ['srp', 'Serbian'],
                    ['srr', 'Serer'], ['ssa', 'Nilo-Saharan languages'],
                    ['ssw', 'Swati'], ['suk', 'Sukuma'],
                    ['sun', 'Sundanese'], ['sus', 'Susu'],
                    ['sux', 'Sumerian'], ['swa', 'Swahili'],
                    ['swe', 'Swedish'], ['syc', 'Classical Syriac'],
                    ['syr', 'Syriac'], ['tah', 'Tahitian'],
                    ['tai', 'Tai languages'], ['tam', 'Tamil'],
                    ['tat', 'Tatar'], ['tel', 'Telugu'],
                    ['tem', 'Timne'], ['ter', 'Tereno'],
                    ['tet', 'Tetum'], ['tgk', 'Tajik'],
                    ['tgl', 'Tagalog'], ['tha', 'Thai'],
                    ['bod', 'Tibetan'], ['tig', 'Tigre'],
                    ['tir', 'Tigrinya'], ['tiv', 'Tiv'],
                    ['tkl', 'Tokelau'], ['tlh', 'Klingon; tlhIngan-Hol'],
                    ['tli', 'Tlingit'], ['tmh', 'Tamashek'],
                    ['tog', 'Tonga (Nyasa)'],
                    ['ton', 'Tonga (Tonga Islands)'], ['tpi', 'Tok Pisin'],
                    ['tsi', 'Tsimshian'], ['tsn', 'Tswana'],
                    ['tso', 'Tsonga'], ['tuk', 'Turkmen'],
                    ['tum', 'Tumbuka'], ['tup', 'Tupi languages'],
                    ['tur', 'Turkish'], ['tut', 'Altaic languages'],
                    ['tvl', 'Tuvalu'], ['twi', 'Twi'],
                    ['tyv', 'Tuvinian'], ['udm', 'Udmurt'],
                    ['uga', 'Ugaritic'], ['uig', 'Uighur; Uyghur'],
                    ['ukr', 'Ukrainian'], ['umb', 'Umbundu'],
                    ['und', 'Undetermined'], ['urd', 'Urdu'],
                    ['uzb', 'Uzbek'], ['vai', 'Vai'],
                    ['ven', 'Venda'], ['vie', 'Vietnamese'],
                    ['vol', 'Volap\xfck'], ['vot', 'Votic'],
                    ['wak', 'Wakashan languages'],
                    ['wal', 'Wolaitta; Wolaytta'], ['war', 'Waray'],
                    ['was', 'Washo'], ['cym', 'Welsh'],
                    ['wen', 'Sorbian languages'], ['wln', 'Walloon'],
                    ['wol', 'Wolof'], ['xal', 'Kalmyk; Oirat'],
                    ['xho', 'Xhosa'], ['yao', 'Yao'],
                    ['yap', 'Yapese'], ['yid', 'Yiddish'],
                    ['yor', 'Yoruba'], ['ypk', 'Yupik languages'],
                    ['zap', 'Zapotec'],
                    ['zbl', 'Blissymbols; Blissymbolics; Bliss'],
                    ['zen', 'Zenaga'],
                    ['zgh', 'Standard Moroccan Tamazight'],
                    ['zha', 'Zhuang; Chuang'], ['znd', 'Zande languages'],
                    ['zul', 'Zulu'], ['zun', 'Zuni'],
                    ['zxx', 'No linguistic content; Not applicable'],
                    ['zza', 'Zaza; Dimili; Dimli; Kirdki; Kirmanjki; Zazaki']
                ]
            ),
            preserve_default=True,
        ),
    ]



from django.db import models
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.models.document_file_page_models import DocumentFilePage
from mayan.apps.documents.models.document_type_models import DocumentType

from .managers import (
    DocumentFilePageContentManager, DocumentTypeSettingsManager
)


class DocumentFilePageContent(models.Model):
    """
    This model store's the parsed content of a document page.
    """
    document_file_page = models.OneToOneField(
        on_delete=models.CASCADE, related_name='content', to=DocumentFilePage,
        verbose_name=_(message='Document file page')
    )
    content = models.TextField(
        blank=True, help_text=_(
            'The actual text content as extracted by the document '
            'parsing backend.'
        ), verbose_name=_(message='Content')
    )

    objects = DocumentFilePageContentManager()

    class Meta:
        verbose_name = _(message='Document file page content')
        verbose_name_plural = _(message='Document file page contents')

    def __str__(self):
        return force_str(s=self.document_file_page)


class DocumentTypeSettings(models.Model):
    """
    This model stores the parsing settings for a document type.
    """
    document_type = models.OneToOneField(
        on_delete=models.CASCADE, related_name='parsing_settings',
        to=DocumentType, unique=True, verbose_name=_(message='Document type')
    )
    auto_parsing = models.BooleanField(
        default=True, help_text=_(
            'Automatically queue newly created documents for parsing.'
        ), verbose_name=_(message='Auto parsing')
    )

    objects = DocumentTypeSettingsManager()

    def natural_key(self):
        return self.document_type.natural_key()
    natural_key.dependencies = ['documents.DocumentType']

    class Meta:
        verbose_name = _(message='Document type settings')
        verbose_name_plural = _(message='Document types settings')



from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.permissions import permission_document_create
from mayan.apps.navigation.classes import Link

from .icons import icon_storage_file_delete, icon_storage_file_select

link_storage_file_delete = Link(
    icon=icon_storage_file_delete, kwargs={
        'source_id': 'source.pk',
        'action_name': '"file_delete"'
    }, permission=permission_document_create, query={
        'document_id': 'document.pk',
        'document_type_id': 'document_type.pk',
        'source_id': 'source.pk',
        'encoded_filename': 'object.encoded_filename'
    }, tags='dangerous', text=_(message='Delete'),
    view='sources:source_action'
)
link_source_file_select = Link(
    html_data={'encoded_filename': 'object.encoded_filename'},
    html_extra_classes='source_stored_files-stored_file-select',
    icon=icon_storage_file_select, text=_(message='Select'), url=''
)



from mayan.apps.source_compressed.source_backends.literals import SOURCE_UNCOMPRESS_CHOICE_NEVER
from mayan.apps.source_periodic.source_backends.literals import DEFAULT_PERIOD_INTERVAL
from mayan.apps.source_stored_files.tests.mixins import SourceTestMixinStoredFile
from mayan.apps.sources.tests.literals import (
    TEST_CASE_ACTION_NAME_SOURCE_CREATE, TEST_CASE_INTERFACE_NAME_MODEL
)

from .literals import TEST_SOURCE_BACKEND_PATH_WATCH_FOLDER


class WatchFolderSourceTestMixin(SourceTestMixinStoredFile):
    _test_source_backend_path = TEST_SOURCE_BACKEND_PATH_WATCH_FOLDER

    def _get_test_source_backend_data(self, action_name, interface_name):
        result = super()._get_test_source_backend_data(
            action_name=action_name, interface_name=interface_name
        )

        if action_name == TEST_CASE_ACTION_NAME_SOURCE_CREATE:
            result.update(
                {
                    'document_type_id': self._test_document_type.pk,
                    'folder_path': self._test_source_temporary_folder,
                    'interval': DEFAULT_PERIOD_INTERVAL,
                    'uncompress': SOURCE_UNCOMPRESS_CHOICE_NEVER
                }
            )
        elif action_name == 'file_delete':
            if interface_name == TEST_CASE_INTERFACE_NAME_MODEL:
                result['encoded_filename'] = self._test_source_stored_file.encoded_filename

        return result



from django.utils.translation import gettext_lazy as _

from mayan.apps.task_manager.classes import CeleryQueue
from mayan.apps.task_manager.workers import worker_d

queue_file_metadata = CeleryQueue(
    label=_(message='File metadata'), name='file_metadata', worker=worker_d
)

queue_file_metadata.add_task_type(
    dotted_path='mayan.apps.file_metadata.tasks.task_document_file_metadata_process',
    label=_(message='Process document file')
)
queue_file_metadata.add_task_type(
    dotted_path='mayan.apps.file_metadata.tasks.task_document_file_metadata_finished',
    label=_(message='Finish document file metadata processing')
)
queue_file_metadata.add_task_type(
    dotted_path='mayan.apps.file_metadata.tasks.task_metadata_driver_process',
    label=_(message='Process file metadata driver')
)



from mayan.apps.converter.layers import layer_saved_transformations
from mayan.apps.converter.permissions import (
    permission_transformation_delete, permission_transformation_edit
)
from mayan.apps.converter.tests.mixins import LayerTestMixin
from mayan.apps.documents.tests.literals import TEST_FILE_MULTI_PAGE_TIFF_FILENAME
from mayan.apps.file_caching.events import event_cache_partition_purged
from mayan.apps.file_caching.models import CachePartitionFile
from mayan.apps.file_caching.permissions import permission_cache_partition_purge
from mayan.apps.file_caching.tests.mixins import CachePartitionViewTestMixin

from ..events import (
    event_document_file_deleted, event_document_file_edited,
    event_document_version_created, event_document_version_edited,
    event_document_version_page_created, event_document_version_page_deleted
)
from ..permissions import (
    permission_document_file_delete, permission_document_file_edit,
    permission_document_file_print, permission_document_file_tools,
    permission_document_file_view
)

from .base import GenericDocumentViewTestCase
from .mixins.document_file_mixins import (
    DocumentFileTestMixin, DocumentFileTransformationTestMixin,
    DocumentFileTransformationViewTestMixin, DocumentFileViewTestMixin
)


class DocumentFileIntrospectViewTestCase(
    DocumentFileViewTestMixin, GenericDocumentViewTestCase
):
    def setUp(self):
        super().setUp()
        for document_file_page in self._test_document_file.pages.all():
            document_file_page.delete()

        self._test_document_file_page_count = self._test_document_file.pages.count()
        self._test_document_file_checksum = self._test_document_file.checksum
        self._test_document_file_size = self._test_document_file.size
        self._test_document_version_page_count = self._test_document_version.pages.count()

        self._test_document_file.checksum = None
        self._test_document_file.size = 0
        self._test_document_file.save()

    def test_document_file_introspect_single_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_introspect_single_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_file.refresh_from_db()
        self._test_document.refresh_from_db()
        self._test_document_version = self._test_document.version_active
        self._test_document_version_page = self._test_document_version.pages.first()

        self.assertEqual(
            self._test_document_file.pages.count(),
            self._test_document_file_page_count
        )
        self.assertEqual(self._test_document_file.size, 0)
        self.assertEqual(self._test_document_file.checksum, None)
        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_version_page_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_introspect_single_view_with_access(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_file_tools
        )

        self._clear_events()

        response = self._request_test_document_file_introspect_single_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_file.refresh_from_db()
        self._test_document.refresh_from_db()
        self._test_document_version = self._test_document.version_active
        self._test_document_version_page = self._test_document_version.pages.first()

        self.assertEqual(
            self._test_document_file.pages.count(),
            self._test_document_file_page_count + 1
        )
        self.assertEqual(
            self._test_document_file.size, self._test_document_file_size
        )
        self.assertEqual(
            self._test_document_file.checksum,
            self._test_document_file_checksum
        )
        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_version_page_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(events[0].verb, event_document_file_edited.id)

        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(events[1].verb, event_document_version_created.id)

        self.assertEqual(events[2].action_object, self._test_document_version)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, self._test_document_version_page)
        self.assertEqual(
            events[2].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[3].action_object, self._test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, self._test_document_version)
        self.assertEqual(events[3].verb, event_document_version_edited.id)

    def test_trashed_document_file_introspect_single_view_with_access(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_file_tools
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_introspect_single_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_file.refresh_from_db()
        self._test_document.refresh_from_db()
        self._test_document_version = self._test_document.version_active
        self._test_document_version_page = self._test_document_version.pages.first()

        self.assertEqual(
            self._test_document_file.pages.count(),
            self._test_document_file_page_count
        )
        self.assertEqual(self._test_document_file.size, 0)
        self.assertEqual(self._test_document_file.checksum, None)
        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_version_page_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_introspect_multiple_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_introspect_multiple_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_file.refresh_from_db()
        self._test_document.refresh_from_db()
        self._test_document_version = self._test_document.version_active
        self._test_document_version_page = self._test_document_version.pages.first()

        self.assertEqual(
            self._test_document_file.pages.count(),
            self._test_document_file_page_count
        )
        self.assertEqual(self._test_document_file.size, 0)
        self.assertEqual(self._test_document_file.checksum, None)
        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_version_page_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_introspect_multiple_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_tools
        )

        self._clear_events()

        response = self._request_test_document_file_introspect_multiple_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_file.refresh_from_db()
        self._test_document.refresh_from_db()
        self._test_document_version = self._test_document.version_active
        self._test_document_version_page = self._test_document_version.pages.first()

        self.assertEqual(
            self._test_document_file.pages.count(),
            self._test_document_file_page_count + 1
        )
        self.assertEqual(
            self._test_document_file.size, self._test_document_file_size
        )
        self.assertEqual(
            self._test_document_file.checksum,
            self._test_document_file_checksum
        )
        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_version_page_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 4)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(events[0].verb, event_document_file_edited.id)

        self.assertEqual(events[1].action_object, self._test_document)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(events[1].verb, event_document_version_created.id)

        self.assertEqual(events[2].action_object, self._test_document_version)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(events[2].target, self._test_document_version_page)
        self.assertEqual(
            events[2].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[3].action_object, self._test_document)
        self.assertEqual(events[3].actor, self._test_case_user)
        self.assertEqual(events[3].target, self._test_document_version)
        self.assertEqual(events[3].verb, event_document_version_edited.id)

    def test_trashed_document_file_introspect_multiple_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_tools
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_introspect_multiple_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_file.refresh_from_db()
        self._test_document.refresh_from_db()
        self._test_document_version = self._test_document.version_active
        self._test_document_version_page = self._test_document_version.pages.first()

        self.assertEqual(
            self._test_document_file.pages.count(),
            self._test_document_file_page_count
        )
        self.assertEqual(self._test_document_file.size, 0)
        self.assertEqual(self._test_document_file.checksum, None)
        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_version_page_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentFileViewTestCase(
    DocumentFileTestMixin, DocumentFileViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_file_delete_no_permission(self):
        first_file = self._test_document.file_latest
        self._upload_test_document_file()

        test_document_file_count = self._test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_delete_view(
            document_file=first_file
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.files.count(), test_document_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_delete_with_access(self):
        first_file = self._test_document.file_latest
        self._upload_test_document_file()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_delete
        )

        test_document_file_count = self._test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_delete_view(
            document_file=first_file
        )
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.files.count(), test_document_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(
            events[0].target, self._test_document_version_list[0]
        )
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document)
        self.assertEqual(events[1].verb, event_document_file_deleted.id)

    def test_trashed_document_file_delete_with_access(self):
        first_file = self._test_document.file_latest
        self._upload_test_document_file()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_delete
        )

        test_document_file_count = self._test_document.files.count()

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_delete_view(
            document_file=first_file
        )
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.files.count(), test_document_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_multiple_delete_no_permission(self):
        self._upload_test_document_file()

        test_document_file_count = self._test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_multiple_delete_view()

        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.files.count(), test_document_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_multiple_delete_with_access(self):
        self._upload_test_document_file()

        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_delete
        )

        test_document_file_count = self._test_document.files.count()

        self._clear_events()

        response = self._request_test_document_file_multiple_delete_view()

        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.files.count(), test_document_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document)
        self.assertEqual(events[1].verb, event_document_file_deleted.id)

    def test_document_file_edit_view_no_permission(self):
        document_file_comment = self._test_document_file.comment

        self._clear_events()

        response = self._request_test_document_file_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_file.refresh_from_db()
        self.assertEqual(
            self._test_document_file.comment,
            document_file_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_edit_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_edit
        )

        document_file_comment = self._test_document_file.comment
        document_file_filename = self._test_document_file.filename

        self._clear_events()

        response = self._request_test_document_file_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_file.refresh_from_db()
        self.assertNotEqual(
            self._test_document_file.comment,
            document_file_comment
        )
        self.assertNotEqual(
            self._test_document_file.filename,
            document_file_filename
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_file)
        self.assertEqual(events[0].verb, event_document_file_edited.id)

    def test_trashed_document_file_edit_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_edit
        )

        document_file_comment = self._test_document_file.comment
        document_file_filename = self._test_document_file.filename

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_file.refresh_from_db()
        self.assertEqual(
            self._test_document_file.comment,
            document_file_comment
        )
        self.assertEqual(
            self._test_document_file.filename,
            document_file_filename
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_list_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_list_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._clear_events()

        response = self._request_test_document_file_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=str(self._test_document_file)
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_file_list_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_file_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_print_form_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_print_form_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_print_form_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_print
        )

        self._clear_events()

        response = self._request_test_document_file_print_form_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_file_print_form_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_print
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_print_form_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_print_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_print_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_print_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_print
        )

        self._clear_events()

        response = self._request_test_document_file_print_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_file_print_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_print
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_file_print_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_properties_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_file_properties_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_properties_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_view
        )

        self._clear_events()

        response = self._request_test_document_file_properties_view()
        self.assertContains(
            response=response, text=self._test_document_file.filename,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_file_properties_view_with_access(self):
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_view
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_file_properties_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentFileTransformationViewTestCase(
    LayerTestMixin, DocumentFileTransformationTestMixin,
    DocumentFileTransformationViewTestMixin, GenericDocumentViewTestCase
):
    _test_document_filename = TEST_FILE_MULTI_PAGE_TIFF_FILENAME

    def test_document_file_transformations_clear_view_no_permission(self):
        self._create_document_file_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()

        self._clear_events()

        response = self._request_test_document_file_transformations_clear_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), transformation_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_transformations_clear_view_with_access(self):
        self._create_document_file_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()

        self.grant_access(
            obj=self._test_document_file,
            permission=permission_transformation_delete
        )

        self._clear_events()

        response = self._request_test_document_file_transformations_clear_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), transformation_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_file_transformations_clear_view_with_access(self):
        self._create_document_file_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()

        self.grant_access(
            obj=self._test_document_file,
            permission=permission_transformation_delete
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_file_transformations_clear_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), transformation_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_multiple_transformations_clear_view_no_permission(self):
        self._create_document_file_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()

        self._clear_events()

        response = self._request_test_document_file_multiple_transformations_clear_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), transformation_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_multiple_transformations_clear_view_with_access(self):
        self._create_document_file_transformation()

        transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()

        self.grant_access(
            obj=self._test_document_file,
            permission=permission_document_file_view
        )
        self.grant_access(
            obj=self._test_document_file,
            permission=permission_transformation_delete
        )

        self._clear_events()

        response = self._request_test_document_file_multiple_transformations_clear_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), transformation_count - 1,
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_transformations_clone_view_no_permission(self):
        self._create_document_file_transformation()

        page_first_transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()
        page_last_transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.last()
        ).count()

        self._clear_events()

        response = self._request_test_document_file_transformations_clone_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), page_first_transformation_count
        )
        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.last()
            ).count(), page_last_transformation_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_transformations_clone_view_with_access(self):
        self._create_document_file_transformation()

        page_first_transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.first()
        ).count()
        page_last_transformation_count = layer_saved_transformations.get_transformations_for(
            obj=self._test_document_file.pages.last()
        ).count()

        self.grant_access(
            obj=self._test_document_file,
            permission=permission_transformation_edit
        )

        self._clear_events()

        response = self._request_test_document_file_transformations_clone_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.first()
            ).count(), page_first_transformation_count
        )
        self.assertEqual(
            layer_saved_transformations.get_transformations_for(
                obj=self._test_document_file.pages.last()
            ).count(), page_last_transformation_count + 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentFileCachePurgeViewTestCase(
    CachePartitionViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_file_cache_purge_no_permission(self):
        self._test_object = self._test_document_file
        self._inject_test_object_content_type()

        self._test_document_file.file_pages.first().generate_image()

        test_document_file_cache_partitions = self._test_document_file.get_cache_partitions()

        cache_partition_file_count = CachePartitionFile.objects.filter(
            partition__in=test_document_file_cache_partitions
        ).count()

        self._clear_events()

        response = self._request_test_object_file_cache_partition_purge_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            CachePartitionFile.objects.filter(
                partition__in=test_document_file_cache_partitions
            ).count(), cache_partition_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_file_cache_purge_with_access(self):
        self._test_object = self._test_document_file
        self._inject_test_object_content_type()

        self.grant_access(
            obj=self._test_document_file,
            permission=permission_cache_partition_purge
        )

        self._test_document_file.file_pages.first().generate_image()

        test_document_file_cache_partitions = self._test_document_file.get_cache_partitions()

        cache_partition_file_count = CachePartitionFile.objects.filter(
            partition__in=test_document_file_cache_partitions
        ).count()

        self._clear_events()

        cache_partitions = self._test_document_file.get_cache_partitions()

        response = self._request_test_object_file_cache_partition_purge_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            CachePartitionFile.objects.filter(
                partition__in=test_document_file_cache_partitions
            ).count(), cache_partition_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, self._test_document_file)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, cache_partitions[0])
        self.assertEqual(events[0].verb, event_cache_partition_purged.id)

        self.assertEqual(events[1].action_object, self._test_document_file)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, cache_partitions[1])
        self.assertEqual(events[1].verb, event_cache_partition_purged.id)



import os

from django.conf import settings

STORAGE_NAME_SIGNATURE_CAPTURES_CACHE = 'signature_captures__signaturecaptures_cache'

DEFAULT_SIGNATURE_CAPTURES_SIGNATURE_CAPTURE_CACHE_MAXIMUM_SIZE = 10 * 2 ** 20  # 10 Megabytes
DEFAULT_SIGNATURE_CAPTURES_SIGNATURE_CAPTURE_CACHE_STORAGE_BACKEND = 'django.core.files.storage.FileSystemStorage'
DEFAULT_SIGNATURE_CAPTURES_SIGNATURE_CAPTURE_CACHE_STORAGE_BACKEND_ARGUMENTS = {
    'location': os.path.join(
        settings.MEDIA_ROOT, 'signature_captures_signature_capture_cache'
    )
}



from django.utils.translation import gettext_lazy as _

from mayan.apps.source_periodic.source_backend_actions.periodic_actions import SourceBackendActionPeriodicDocumentUpload
from mayan.apps.source_periodic.source_backends.mixins import SourceBackendMixinPeriodicCompressed
from mayan.apps.source_stored_files.source_backends.filesystem_source_mixins import SourceBackendMixinStoredFileLocationFilesystem
from mayan.apps.source_stored_files.source_backends.stored_file_source_mixins import SourceBackendMixinStoredFileInteractiveNot
from mayan.apps.sources.source_backends.base import SourceBackend
from mayan.apps.sources.source_backends.mixins import SourceBackendMixinRegularExpression


class SourceBackendWatchFolder(
    SourceBackendMixinStoredFileLocationFilesystem,
    SourceBackendMixinPeriodicCompressed,
    SourceBackendMixinRegularExpression,
    SourceBackendMixinStoredFileInteractiveNot, SourceBackend
):
    action_class_list = (SourceBackendActionPeriodicDocumentUpload,)
    label = _(message='Watch folder')



from django.urls import reverse
from django.utils.translation import gettext_lazy as _

from mayan.apps.common.utils import parse_range
from mayan.apps.converter.transformations import TransformationResize
from mayan.apps.views.generics import FormView, SimpleView
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..forms.misc_forms import PrintForm
from ..literals import PAGE_RANGE_RANGE
from ..settings import setting_print_height, setting_print_width


class PrintFormView(ExternalObjectViewMixin, FormView):
    external_object_class = None
    external_object_permission = None
    external_object_pk_url_kwarg = None
    form_class = PrintForm
    print_view_name = None
    print_view_kwarg = None

    def _add_recent_document(self):
        self.external_object.add_as_recent_document_for_user(
            user=self.request.user
        )

    def dispatch(self, request, *args, **kwargs):
        result = super().dispatch(request, *args, **kwargs)

        self._add_recent_document()

        return result

    def get_extra_context(self):
        return {
            'form_action': reverse(
                viewname=self.print_view_name, kwargs={
                    self.print_view_kwarg: self.external_object.pk
                }
            ),
            'object': self.external_object,
            'submit_method': 'GET',
            'submit_target': '_blank',
            'title': _(message='Print: %s') % self.external_object
        }


class DocumentPrintBaseView(ExternalObjectViewMixin, SimpleView):
    external_object_class = None
    external_object_permission = None
    external_object_pk_url_kwarg = None
    template_name = 'documents/document_print.html'

    def dispatch(self, request, *args, **kwargs):
        result = super().dispatch(request, *args, **kwargs)

        self._add_recent_document()

        return result

    def get_extra_context(self):
        page_group = self.request.GET.get('page_group', None)
        page_range = self.request.GET.get('page_range', None)

        transformation_instance_list = (
            TransformationResize(
                height=setting_print_height.value,
                width=setting_print_width.value
            ),
        )

        if page_group == PAGE_RANGE_RANGE:
            if page_range:
                page_range = parse_range(range_string=page_range)
                pages = self.external_object.pages.filter(
                    page_number__in=page_range
                )
            else:
                pages = self.external_object.pages.all()
        else:
            pages = self.external_object.pages.all()

        return {
            'appearance_type': 'plain',
            'pages': pages,
            'transformation_instance_list': transformation_instance_list
        }



import json

from django.db import models


class SourceManager(models.Manager):
    def create_backend(self, label, backend_path, backend_data=None):
        self.create(
            backend_path=backend_path, backend_data=json.dumps(
                obj=backend_data or {}
            ), label=label
        )



from mayan.apps.rest_api import generics

from .models import Announcement
from .permissions import (
    permission_announcement_create, permission_announcement_delete,
    permission_announcement_edit, permission_announcement_view
)
from .serializers import AnnouncementSerializer


class APIAnnouncementListView(generics.ListCreateAPIView):
    """
    get: Returns a list of all the announcements.
    post: Create a new announcement.
    """
    mayan_object_permission_map = {'GET': permission_announcement_view}
    mayan_view_permission_map = {'POST': permission_announcement_create}
    ordering_fields = (
        'enabled', 'end_datetime', 'id', 'label', 'start_datetime'
    )
    serializer_class = AnnouncementSerializer
    source_queryset = Announcement.objects.all()

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }


class APIAnnouncementView(generics.RetrieveUpdateDestroyAPIView):
    """
    delete: Delete the selected announcement.
    get: Return the details of the selected announcement.
    patch: Edit the selected announcement.
    put: Edit the selected announcement.
    """
    lookup_url_kwarg = 'announcement_id'
    mayan_object_permission_map = {
        'DELETE': permission_announcement_delete,
        'GET': permission_announcement_view,
        'PATCH': permission_announcement_edit,
        'PUT': permission_announcement_edit
    }
    serializer_class = AnnouncementSerializer
    source_queryset = Announcement.objects.all()

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }



DEFAULT_VIEWS_PAGINATE_BY = 30
DEFAULT_VIEWS_PAGING_ARGUMENT = 'page'
DEFAULT_VIEWS_SHOW_DROPZONE_SUBMIT_BUTTON = False

LIST_MODE_CHOICE_LIST = 'list'
LIST_MODE_CHOICE_ITEM = 'item'

PK_LIST_SEPARATOR = ','

TEXT_LIST_AS_ITEMS_PARAMETER = '_list_mode'
TEXT_LIST_AS_ITEMS_VARIABLE_NAME = 'list_as_items'
TEXT_CHOICE_ITEMS = 'items'
TEXT_CHOICE_LIST = 'list'

TEXT_SORT_FIELD_PARAMETER = '_ordering'
TEXT_SORT_FIELD_VARIABLE_NAME = 'ordering'



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('document_indexing', '0016_auto_20191005_0647')
    ]

    operations = [
        migrations.AlterField(
            model_name='indexinstancenode',
            name='level',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='indexinstancenode',
            name='lft',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='indexinstancenode',
            name='rght',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='indextemplatenode',
            name='level',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='indextemplatenode',
            name='lft',
            field=models.PositiveIntegerField(editable=False),
        ),
        migrations.AlterField(
            model_name='indextemplatenode',
            name='rght',
            field=models.PositiveIntegerField(editable=False),
        )
    ]



from mayan.apps.appearance.classes import Icon

icon_credential_backend_selection = Icon(
    driver_name='fontawesome-dual', primary_symbol='id-card',
    secondary_symbol='plus'
)
icon_credential_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='id-card',
    secondary_symbol='plus'
)
icon_credential_delete = Icon(
    driver_name='fontawesome', symbol='times'
)
icon_credential_edit = Icon(
    driver_name='fontawesome', symbol='pencil-alt'
)
icon_credential_list = Icon(
    driver_name='fontawesome', symbol='id-card'
)
icon_credential_setup = Icon(
    driver_name='fontawesome', symbol='id-card'
)
icon_credentials = Icon(
    driver_name='fontawesome', symbol='id-card'
)



from django.utils.translation import gettext_lazy as _

from rest_framework.generics import get_object_or_404

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.serializers.document_serializers import DocumentSerializer
from mayan.apps.rest_api import serializers
from mayan.apps.user_management.serializers import UserSerializer

from .models import DocumentCheckout
from .permissions import permission_document_check_out


class DocumentCheckoutSerializer(serializers.ModelSerializer):
    document = DocumentSerializer(
        label=_(message='Document')
    )
    user = UserSerializer(
        label=_(message='User')
    )

    class Meta:
        extra_kwargs = {
            'url': {
                'label': _(message='URL'),
                'lookup_url_kwarg': 'checkout_id',
                'view_name': 'rest_api:checkedout-document-view'
            }
        }
        fields = (
            'checkout_datetime', 'document', 'expiration_datetime', 'id',
            'url', 'user'
        )
        model = DocumentCheckout
        read_only_fields = (
            'checkout_datetime', 'document', 'id', 'url'
        )


class NewDocumentCheckoutSerializer(serializers.ModelSerializer):
    block_new_file = serializers.BooleanField()
    document_pk = serializers.IntegerField(
        help_text=_(message='Primary key of the document to be checked out.'),
        label=_(message='Document ID'), write_only=True
    )
    expiration_datetime = serializers.DateTimeField()

    class Meta:
        fields = (
            'block_new_file', 'document', 'document_pk',
            'expiration_datetime', 'id',
        )
        model = DocumentCheckout
        read_only_fields = ('document',)
        write_only_fields = ('document_pk',)

    def create(self, validated_data):
        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_document_check_out,
            queryset=Document.valid.all(),
            user=self.context['request'].user
        )

        document = get_object_or_404(
            queryset=queryset, pk=validated_data.pop('document_pk')
        )

        validated_data['document'] = document
        validated_data['user'] = self.context['request'].user
        return super().create(validated_data=validated_data)



from .events import event_user_logged_in, event_user_logged_out


def handler_user_logged_in(sender, **kwargs):
    event_user_logged_in.commit(
        actor=kwargs['user'], target=kwargs['user']
    )


def handler_user_logged_out(sender, **kwargs):
    event_user_logged_out.commit(
        actor=kwargs['user'], target=kwargs['user']
    )



from django.utils.translation import gettext_lazy as _

from mayan.apps.documents.models.document_models import Document

from ..icons import icon_document_link_send_single

from .base import MailingObjectLinkSendView


class MailDocumentLinkView(MailingObjectLinkSendView):
    pk_url_kwarg = 'document_id'
    source_queryset = Document.valid.all()
    success_message = _(
        message='%(count)d document link queued for email delivery'
    )
    success_message_plural = _(
        '%(count)d document links queued for email delivery'
    )
    title = 'Email document link'
    title_document = 'Email link for document: %s'
    title_plural = 'Email document links'
    view_icon = icon_document_link_send_single



import logging

from django.template import Library

from ..literals import MESSAGE_SQLITE_WARNING
from ..utils import check_for_sqlite

logger = logging.getLogger(name=__name__)
register = Library()


@register.simple_tag
def databases_check_sqlite():
    if check_for_sqlite():
        return MESSAGE_SQLITE_WARNING



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0058_auto_20200916_1146'),
        ('checkouts', '0009_rename_documentcheckout_field')
    ]

    operations = [
        migrations.RenameModel(
            old_name='NewVersionBlock',
            new_name='NewFileBlock'
        ),
        migrations.AlterModelOptions(
            name='newfileblock',
            options={
                'verbose_name': 'New file block',
                'verbose_name_plural': 'New file blocks'
            }
        ),
        migrations.AlterField(
            model_name='documentcheckout',
            name='block_new_file',
            field=models.BooleanField(
                default=True, help_text='Do not allow new file of this '
                'document to be uploaded.',
                verbose_name='Block new file upload'
            )
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0034_auto_20160509_2321')
    ]

    operations = [
        migrations.CreateModel(
            name='DocumentPageCachedImage',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'filename', models.CharField(
                        max_length=128, verbose_name='Filename'
                    )
                )
            ],
            options={
                'verbose_name': 'Document page cached image',
                'verbose_name_plural': 'Document page cached images'
            }
        ),
        migrations.CreateModel(
            name='DocumentPageResult',
            fields=[],
            options={
                'ordering': ('document_version__document', 'page_number'),
                'verbose_name': 'Document page',
                'proxy': True,
                'verbose_name_plural': 'Document pages'
            },
            bases=('documents.documentpage',)
        ),
        migrations.AddField(
            model_name='documentpagecachedimage',
            name='document_page',
            field=models.ForeignKey(
                on_delete=models.CASCADE, related_name='cached_images',
                to='documents.DocumentPage', verbose_name='Document page'
            )
        )
    ]



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('ocr', '0007_auto_20170827_1617')
    ]

    operations = [
        migrations.AlterField(
            model_name='documentpageocrcontent',
            name='content',
            field=models.TextField(
                blank=True, help_text='The actual text content extracted by '
                'the OCR backend.', verbose_name='Content'
            )
        )
    ]
    run_before = [
        ('documents', '0057_auto_20200916_1057')
    ]



from django.conf import settings
from django.utils.translation import gettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig
from mayan.apps.common.menus import menu_user

from .links import link_current_user_oidc_details, separator_oidc_user


class AuthenticationOIDCApp(MayanAppConfig):
    app_namespace = 'authentication_oidc'
    app_url = ''
    has_rest_api = False
    has_static_media = True
    has_tests = False
    name = 'mayan.apps.authentication_oidc'
    verbose_name = _(message='Authentication OIDC')

    def ready(self):
        super().ready()

        settings.STRONGHOLD_PUBLIC_URLS += (r'^/oidc.*',)

        menu_user.bind_links(
            links=(
                separator_oidc_user, link_current_user_oidc_details
            )
        )



import logging

from django.apps import apps

from mayan.apps.converter.layers import layer_saved_transformations

logger = logging.getLogger(name=__name__)


class SourceBusinessLogicMixin:
    @staticmethod
    def callback_post_document_create(document, **kwargs):
        Source = apps.get_model(app_label='sources', model_name='Source')

        source = Source.objects.get(
            pk=kwargs['source_id']
        )

        source_backend_instance = source.get_backend_instance()

        source_backend_instance.callback_post_document_create(
            document=document, **kwargs
        )

    @staticmethod
    def callback_post_document_file_create(document_file, **kwargs):
        Source = apps.get_model(app_label='sources', model_name='Source')

        source = Source.objects.get(
            pk=kwargs['source_id']
        )

        source_backend_instance = source.get_backend_instance()

        source_backend_instance.callback_post_document_file_create(
            document_file=document_file, **kwargs
        )

    @staticmethod
    def callback_post_document_file_upload(document_file, **kwargs):
        Source = apps.get_model(app_label='sources', model_name='Source')

        source = Source.objects.get(
            pk=kwargs['source_id']
        )

        layer_saved_transformations.copy_transformations(
            source=source, targets=document_file.pages.all()
        )

        source_backend_instance = source.get_backend_instance()

        source_backend_instance.callback_post_document_file_upload(
            document_file=document_file, **kwargs
        )

    def action_execute(
        self, name, interface_name, interface_load_kwargs=None,
        interface_retrieve_kwargs=None
    ):
        action = self.get_action(name=name)

        return action.execute(
            interface_name=interface_name,
            interface_load_kwargs=interface_load_kwargs,
            interface_retrieve_kwargs=interface_retrieve_kwargs
        )

    @property
    def actions(self):
        # Used by the REST API serializer.
        return self.get_action_list()

    def fullname(self):
        return '{} {}'.format(
            self.get_backend_class_label(), self.label
        )

    def get_action(self, name):
        backend_instance = self.get_backend_instance()
        return backend_instance.get_action(name=name)

    def get_action_list(self):
        backend_instance = self.get_backend_instance()
        return backend_instance.get_action_list()



from rest_framework import status

from mayan.apps.documents.api_views.api_view_mixins import (
    ParentObjectDocumentAPIViewMixin
)
from mayan.apps.rest_api import generics

from .permissions import permission_document_version_export
from .tasks import task_document_version_export


class APIDocumentVersionExportView(
    ParentObjectDocumentAPIViewMixin, generics.ObjectActionAPIView
):
    """
    post: Exports the specified document version.
    """
    action_response_status = status.HTTP_202_ACCEPTED
    lookup_url_kwarg = 'document_version_id'
    mayan_object_permission_map = {
        'POST': permission_document_version_export
    }

    def get_source_queryset(self):
        return self.get_document().versions.all()

    def object_action(self, obj, request, serializer):
        task_document_version_export.apply_async(
            kwargs={
                'document_version_id': obj.pk,
                'user_id': request.user.id
            }
        )



from django.contrib.auth import get_user_model

from mayan.apps.common.tests.mixins import ManagementCommandTestMixin
from mayan.apps.testing.tests.base import BaseTestCase

from ..literals import COMMAND_NAME_AUTOADMIN_CREATE
from ..models import AutoAdminSingleton


class AutoAdminManagementCommandTestCase(
    ManagementCommandTestMixin, BaseTestCase
):
    _test_management_command_name = COMMAND_NAME_AUTOADMIN_CREATE
    create_test_case_user = False

    def tearDown(self):
        AutoAdminSingleton.objects.all().delete()
        super().tearDown()

    def test_autoadmin_creation(self):
        self._call_test_management_command()

        autoadmin = AutoAdminSingleton.objects.get()
        user = get_user_model().objects.first()

        self.assertEqual(AutoAdminSingleton.objects.count(), 1)

        self.assertEqual(autoadmin.account, user)
        self.assertEqual(autoadmin.account.email, user.email)
        self.assertEqual(autoadmin.password_hash, user.password)



from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _, ngettext

from mayan.apps.acls.models import AccessControlList
from mayan.apps.organizations.utils import get_organization_installation_url
from mayan.apps.views.generics import MultipleObjectFormActionView

from ..classes import ModelMailingAction
from ..exceptions import MailerError
from ..forms import ObjectMailForm
from ..models import UserMailer
from ..permissions import permission_mailing_profile_use
from ..tasks import task_send_object


class MailingObjectSendView(MultipleObjectFormActionView):
    form_class = ObjectMailForm

    def get_as_attachment(self):
        model_mailing_action = self.get_model_mailing_action()

        as_attachment = model_mailing_action.as_attachment

        return as_attachment

    def get_extra_context(self):
        queryset = self.object_list

        result = {
            'title': ngettext(
                singular=self.title,
                plural=self.title_plural,
                number=queryset.count()
            )
        }

        if queryset.count() == 1:
            result.update(
                {
                    'object': queryset.first(),
                    'title': _(
                        message=self.title_document
                    ) % queryset.first()
                }
            )

        return result

    def get_form_extra_kwargs(self):
        return {
            'as_attachment': self.get_as_attachment(),
            'user': self.request.user
        }

    def get_mailing_action_name(self):
        raise NotImplementedError

    def get_model_mailing_action(self):
        action_name = self.get_mailing_action_name()
        model = self.source_queryset.model

        try:
            model_mailing_action = ModelMailingAction.get_action_for_model(
                action_name=action_name, model=model
            )
        except MailerError as exception:
            raise ValidationError(
                message=str(exception)
            )
        else:
            return model_mailing_action

    def get_object_permission(self):
        model_mailing_action = self.get_model_mailing_action()

        permission = model_mailing_action.kwargs['permission']

        return permission

    def object_action(self, form, instance):
        as_attachment = self.get_as_attachment()

        queryset = AccessControlList.objects.restrict_queryset(
            permission=permission_mailing_profile_use,
            queryset=UserMailer.objects.filter(enabled=True),
            user=self.request.user
        )

        mailing_profile = get_object_or_404(
            klass=queryset, pk=form.cleaned_data['mailing_profile'].pk
        )

        content_type = ContentType.objects.get_for_model(model=instance)

        organization_installation_url = get_organization_installation_url(
            request=self.request
        )

        try:
            object_name = instance._meta.model._meta.verbose_name
        except AttributeError:
            object_name = _('Object')

        kwargs = {
            'as_attachment': as_attachment,
            'body': form.cleaned_data['body'],
            'content_type_id': content_type.pk,
            'object_id': instance.pk,
            'object_name': object_name,
            'organization_installation_url': organization_installation_url,
            'recipient': form.cleaned_data['email'],
            'sender': self.request.user.email,
            'subject': form.cleaned_data['subject'],
            'mailing_profile_id': mailing_profile.pk,
            'user_id': self.request.user.pk
        }

        task_send_object.apply_async(kwargs=kwargs)


class MailingObjectAttachmentSendView(MailingObjectSendView):
    def get_mailing_action_name(self):
        return 'attachment'


class MailingObjectLinkSendView(MailingObjectSendView):
    def get_mailing_action_name(self):
        return 'link'



from django.core import management

from ..base import CommonAppManagementCommand


class Command(management.BaseCommand):
    help = 'Initializes an install and gets it ready to be used.'

    def add_arguments(self, parser):
        parser.add_argument(
            '--force', action='store_true', dest='force',
            help='Force execution of the initialization process.',
        )
        parser.add_argument(
            '--no-dependencies', action='store_true', dest='no_dependencies',
            help='Don\'t download dependencies.',
        )

    def handle(self, *args, **options):
        instance = CommonAppManagementCommand()

        try:
            instance.do_initial_setup(
                force=options.get('force', False),
                no_dependencies=options.get('no_dependencies', False)
            )
        except Exception as exception:
            self.stderr.write(
                msg=self.style.NOTICE(
                    str(exception)
                )
            )
            exit(1)



class OwnerPlusFilteredQuerysetAPIViewMixin:
    def filter_queryset(self, queryset):
        user = self.request.user

        if user.is_authenticated:
            queryset_user_owned = queryset.filter(user=self.request.user)
        else:
            queryset_user_owned = queryset.none()

        queryset = super().filter_queryset(queryset=queryset)

        queryset = queryset_user_owned | queryset

        return queryset.distinct()



from drf_yasg.views import get_schema_view

from rest_framework import mixins, permissions, renderers
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.schemas.generators import EndpointEnumerator

import mayan
from mayan.apps.organizations.settings import setting_organization_url_base_path
from mayan.apps.rest_api import generics

from .classes import BatchRequestCollection, Endpoint
from .generics import ListAPIView, RetrieveAPIView
from .schemas import openapi_info
from .serializers import (
    BatchAPIRequestResponseSerializer, EndpointSerializer,
    ProjectInformationSerializer
)


class APIRoot(ListAPIView):
    serializer_class = EndpointSerializer
    swagger_schema = None

    def get_source_queryset(self):
        """
        get: Return a list of all API root endpoints. This includes the
        API version root and root services.
        """
        endpoint_api_version = Endpoint(
            label='API version root', viewname='rest_api:api_version_root'
        )
        endpoint_redoc = Endpoint(
            label='ReDoc UI', viewname='rest_api:schema-redoc'
        )
        endpoint_swagger = Endpoint(
            label='Swagger UI', viewname='rest_api:schema-swagger-ui'
        )
        endpoint_swagger_schema_json = Endpoint(
            label='API schema (JSON)', viewname='rest_api:schema-json',
            kwargs={'format': '.json'}
        )
        endpoint_swagger_schema_yaml = Endpoint(
            label='API schema (YAML)', viewname='rest_api:schema-json',
            kwargs={'format': '.yaml'}
        )
        return [
            endpoint_api_version,
            endpoint_swagger,
            endpoint_redoc,
            endpoint_swagger_schema_json,
            endpoint_swagger_schema_yaml
        ]


class APIVersionRoot(ListAPIView):
    serializer_class = EndpointSerializer
    swagger_schema = None

    def get_source_queryset(self):
        """
        get: Return a list of the API version resources and endpoint.
        """
        endpoint_enumerator = EndpointEnumerator()

        if setting_organization_url_base_path.value:
            url_index = 4
        else:
            url_index = 3

        # Extract the resource names from the API endpoint URLs
        parsed_urls = set()
        for entry in endpoint_enumerator.get_api_endpoints():
            try:
                url = entry[0].split('/')[url_index]
            except IndexError:
                """An unknown or invalid URL"""
            else:
                parsed_urls.add(url)

        endpoints = []
        for url in sorted(parsed_urls):
            if url:
                endpoints.append(
                    Endpoint(label=url)
                )

        return endpoints


class BatchRequestAPIView(mixins.ListModelMixin, generics.GenericAPIView):
    """
    post: Submit a batch API request.
    """
    serializer_class = BatchAPIRequestResponseSerializer

    def post(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)

    def get_source_queryset(self):
        serializer = self.get_serializer(data=self.request.data)
        serializer.is_valid(raise_exception=True)
        batch_request_collection = BatchRequestCollection(
            request_list=serializer.validated_data.get('requests')
        )
        return batch_request_collection.execute(
            view_request=self.request._request
        )


class BrowseableObtainAuthToken(ObtainAuthToken):
    """
    Obtain an API authentication token.
    """
    renderer_classes = (
        renderers.BrowsableAPIRenderer, renderers.JSONRenderer
    )


class ProjectInformationAPIView(RetrieveAPIView):
    serializer_class = ProjectInformationSerializer

    def get_object(self):
        return mayan


schema_view = get_schema_view(
    info=openapi_info,
    public=True,
    permission_classes=(permissions.AllowAny,),
    validators=['flex', 'ssv']
)



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('documents', '0057_auto_20200916_1057')
    ]

    operations = [
        migrations.AlterModelOptions(
            name='documentfile',
            options={
                'ordering': ('timestamp',), 'verbose_name': 'Document file',
                'verbose_name_plural': 'Document files'
            }
        ),
        migrations.AlterField(
            model_name='documentfile',
            name='comment',
            field=models.TextField(
                blank=True, default='', help_text='An optional short text '
                'describing the document file.', verbose_name='Comment'
            )
        ),
        migrations.AlterField(
            model_name='documentfile',
            name='document',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='files', to='documents.Document',
                verbose_name='Document'
            )
        ),
        migrations.AlterField(
            model_name='documentfile',
            name='encoding',
            field=models.CharField(
                blank=True, editable=False, help_text='The document file '
                'file encoding. binary 7-bit, binary 8-bit, text, base64, '
                'etc.', max_length=64, null=True, verbose_name='Encoding'
            )
        ),
        migrations.AlterField(
            model_name='documentfile',
            name='mimetype',
            field=models.CharField(
                blank=True, editable=False, help_text='The document '
                'file\'s file mimetype. MIME types are a standard way to '
                'describe the format of a file, in this case the file '
                'format of the document. Some examples: "text/plain" or '
                '"image/jpeg". ', max_length=255, null=True,
                verbose_name='MIME type'
            )
        ),
        migrations.AlterField(
            model_name='documentfile',
            name='timestamp',
            field=models.DateTimeField(
                auto_now_add=True, db_index=True, help_text='The server '
                'date and time when the document file was processed.',
                verbose_name='Timestamp'
            )
        ),
        migrations.AlterField(
            model_name='documentpage',
            name='document_file',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='file_pages', to='documents.DocumentFile',
                verbose_name='Document file'
            )
        )
    ]
